/**************************************************************************************/
/**                                                                                \n**/
/**                d  a  i  l  y  c  l  i  m  a  t  e  .  c                        \n**/
/**                                                                                \n**/
/**     C implementation of LPJmL                                                  \n**/
/**                                                                                \n**/
/**     Function dailyclimate gets daily value for temperature,                    \n**/
/**     precipitation, cloudiness and wet days.                                    \n**/
/**     The pointer to the climate data has to be initialized by the               \n**/
/**     function getclimate for the specified year.                                \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**                                                                                \n**/
/**************************************************************************************/

#include "lpj.h"

#ifdef COUPLING_WITH_FMS
void dailyclimate(Dailyclimate *daily,    /**< */
                  const Climate *climate, /**< climate data pointer */
                  Climbuf *climbuf,       /**< climate buffer pointer*/
                  Cell *grid,
                  int cell,               /**< cell index */
                  int day,                /**< day (1..365) */
                  int month,              /**< month (0..11) */
                  int dayofmonth,         /**< day of the month */
                  Bool with_anomaly       /**< use anomalies for precipitation */
                  )                       /** \return void */
#else 
void dailyclimate(Dailyclimate *daily,    /**< */
                  const Climate *climate, /**< climate data pointer */
                  Climbuf *climbuf,       /**< climate buffer pointer*/
                  int cell,               /**< cell index */
                  int day,                /**< day (1..365) */
                  int month,              /**< month (0..11) */
                  int dayofmonth,         /**< day of the month */
                  Bool with_anomaly       /**< use anomalies for precipitation */
                  )                       /** \return void */
#endif
{
  Real tamp;
  if(climate->file_temp.fmt!=FMS)
  {
    daily->isdailytemp=isdaily(climate->file_temp);
    if(isdaily(climate->file_temp))
    {
      daily->temp=climate->data.temp[cell*NDAYYEAR+day-1];
      climbuf->mtemp+=daily->temp;
    }
    else
      daily->temp=interpolate(getcelltemp(climate,cell),month,dayofmonth);
  }
  else
  {
    daily->isdailytemp=TRUE;
    climbuf->mtemp+=daily->temp;
  }
#ifdef COUPLING_WITH_FMS
  if(climate->file_qflux.fmt!=FMS)
  {
    if(isdaily(climate->file_qflux))
      daily->qflux=climate->data.qflux[cell*NDAYYEAR+day-1];
    else
      daily->qflux=interpolate(getcellqflux(climate,cell),month,dayofmonth);
  }
  if(climate->file_dedq.fmt!=FMS)
  {
    if(isdaily(climate->file_dedq))
      daily->dedq=climate->data.dedq[cell*NDAYYEAR+day-1];
    else
      daily->dedq=interpolate(getcelldedq(climate,cell),month,dayofmonth);
  }
  if(climate->file_q_ca.fmt!=FMS)
  {
    if(isdaily(climate->file_q_ca))
      daily->q_ca=climate->data.q_ca[cell*NDAYYEAR+day-1];
    else
      daily->q_ca=interpolate(getcellq_ca(climate,cell),month,dayofmonth);
  }
 // else
  //  daily->q_ca=9999.0;
#endif
  if(climate->data.sun!=NULL)
    daily->sun=isdaily(climate->file_cloud) ? climate->data.sun[cell*NDAYYEAR+day-1]
                      : interpolate(getcellsun(climate,cell),month,dayofmonth);
  if(climate->data.lwnet!=NULL)
    daily->lwnet=isdaily(climate->file_lwnet) ? climate->data.lwnet[cell*NDAYYEAR+day-1]
                      : interpolate(getcelllwnet(climate,cell),month,dayofmonth);
  if(climate->data.swdown!=NULL)
    daily->swdown=isdaily(climate->file_swdown) ? climate->data.swdown[cell*NDAYYEAR+day-1]
                      : interpolate(getcellswdown(climate,cell),month,dayofmonth);
  if(climate->data.humid!=NULL)
    daily->humid=isdaily(climate->file_humid) ? climate->data.humid[cell*NDAYYEAR+day-1]
                      : interpolate(getcellhumid(climate,cell),month,dayofmonth);
  if(climate->data.wind!=NULL)
  {
    if(isdaily(climate->file_wind))
      daily->windspeed=climate->data.wind[cell*NDAYYEAR+day-1];
    else
      daily->windspeed=interpolate(getcellwind(climate,cell),month,dayofmonth);
  }
  if(climate->data.tamp!=NULL)
  {
    if(climate->data.tmax!=NULL)
    {
      daily->tmin=isdaily(climate->file_tamp) ?  climate->data.tamp[cell*NDAYYEAR+day-1] : interpolate(getcelltamp(climate,cell),month,dayofmonth);
      daily->tmax=isdaily(climate->file_tmax) ?  climate->data.tmax[cell*NDAYYEAR+day-1] : interpolate(getcelltmax(climate,cell),month,dayofmonth);
    }
    else
    {
      tamp=isdaily(climate->file_tamp) ?  climate->data.tamp[cell*NDAYYEAR+day-1] : interpolate(getcelltamp(climate,cell),month,dayofmonth);
      daily->tmin=daily->temp-tamp*0.5;
      daily->tmax=daily->temp+tamp*0.5;
    }
  }
  if(climate->data.no3deposition!=NULL)
  {
    if(isdaily(climate->file_no3deposition))
      daily->no3deposition=climate->data.no3deposition[cell*NDAYYEAR+day-1];
    else
      daily->no3deposition=interpolate(getcellno3deposition(climate,cell),month,dayofmonth);
  }
  if(climate->data.nh4deposition!=NULL)
  {
    if(isdaily(climate->file_nh4deposition))
      daily->nh4deposition=climate->data.nh4deposition[cell*NDAYYEAR+day-1];
    else
      daily->nh4deposition=interpolate(getcellnh4deposition(climate,cell),month,dayofmonth);
  }
  if(climate->data.lightning!=NULL)
  {
    if(isdaily(climate->file_lightning))
      daily->lightning=climate->data.lightning[cell*NDAYYEAR+day-1];
    else
      daily->lightning=interpolate(getcelllightning(climate,cell),month,dayofmonth)*ndaymonth1[month];
  }
  if(climate->file_prec.fmt!=FMS)
  {
    if(isdaily(climate->file_prec))
    {
#ifdef COUPLING_WITH_FMS
      if(with_anomaly)
        daily->prec=climate->data.prec[cell*NDAYYEAR+day-1]*grid->aprec_poem_last/grid->aprec_ref;
        /* daily->prec=climate->data.prec[cell*NDAYYEAR+day-1]-climate->data.prec_ref[cell*NDAYYEAR+day-1]; absolute delta correction */
      else
#endif
        daily->prec=climate->data.prec[cell*NDAYYEAR+day-1];
      climbuf->mprec+=daily->prec;
    }
    else
    {
      daily->prec=(israndomprec(climate)) ? climbuf->dval_prec[dayofmonth+1] :
          interpolate(getcellprec(climate,cell),month,dayofmonth)*
                        ndaymonth1[month];
      daily->prec=(daily->prec>0.000001) ? daily->prec : 0.0;
    }
  }
  else
    climbuf->mprec+=daily->prec;
 
  if(climate->data.burntarea!=NULL)
   {
    daily->burntarea=isdaily(climate->file_burntarea) ? climate->data.burntarea[cell*NDAYYEAR+day-1]
                     : climate->data.burntarea[cell*12+month] * ndaymonth1[month]; 
    daily->burntarea=(daily->burntarea > 0.000001) ? daily->burntarea : 0.0;
  }
} /* of 'dailyclimate' */
