/**************************************************************************************/
/**                                                                                \n**/
/**                d  a  i  l  y  _  n  a  t  u  r  a  l  .  c                     \n**/
/**                                                                                \n**/
/**     C implementation of LPJmL                                                  \n**/
/**                                                                                \n**/
/**     Function of daily update of natural stand                                  \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**                                                                                \n**/
/**************************************************************************************/

#include "lpj.h"
#include "natural.h"

Real daily_natural(Stand *stand, /**< stand pointer */
                   Real *gc_stand,
                   Real co2,     /**< atmospheric CO2 (ppmv) */
                   const Dailyclimate *climate, /**< Daily climate values */
                   int day,    /**< day (1..365) */
                   Real daylength, /**< length of day (h) */
                   const Real gp_pft[], /**< pot. canopy conductance for PFTs & CFTs*/
                   Real gtemp_air,  /**< value of air temperature response function */
                   Real gtemp_soil, /**< value of soil temperature response function */
                   Real gp_stand,   /* potential stomata conductance */
                   Real gp_stand_leafon, /**< pot. canopy conduct.at full leaf cover */
                   Real eeq,   /**< equilibrium evapotranspiration (mm) */
                   Real par,   /**< photosynthetic active radiation flux */
                   Real melt,  /**< melting water (mm) */
                   int npft,   /**< number of natural PFTs */
                   int ncft, /**< number of crop PFTs   */
                   int year,         /**< simulation year (AD) */
                   Bool withdailyoutput, /**< daily output enabled */
                   Bool UNUSED(intercrop), /**< enabled intercropping */
                   const Config *config /**< LPJ config */
                  ) /** \return runoff (mm) */
{
  int p,l;
#ifdef PERMUTE
  int *pvec;
#endif
  Pft *pft;
  Output *output;
  Real aet_stand[LASTLAYER];
  Real green_transp[LASTLAYER];
  Real evap,evap_blue; /* evaporation (mm) */
  Real *wet; /* wet from pftlist */
  Real rd,gpp,frac_g_evap,runoff,wet_all;
  Real return_flow_b; /* irrigation return flows from surface runoff, lateral runoff and percolation (mm)*/
  Real cover_stand,intercep_stand;
  Real npp; /* net primary productivity (gC/m2) */
  Real wdf; /* water deficit fraction */
  Real gc_pft;
  Bool isphen;
  Real gc;
  Real ge,gi;  /* soil & interception conductance */
#ifdef DAILY_ESTABLISHMENT
  Stocks flux_estab = {0,0};
#endif
  Soil *soil;
  soil = &stand->soil;
  output=&stand->cell->output;
  *gc_stand=0;
  ge=gi=0.0;
  evap=evap_blue=cover_stand=intercep_stand=wet_all=0;

  runoff=return_flow_b=0.0;
  stand->growing_days++;
  if(getnpft(&stand->pftlist)>0)
  {
    wet=newvec(Real,getnpft(&stand->pftlist)); /* wet from pftlist */
    check(wet);
#ifdef PERMUTE
    pvec=newvec(int,getnpft(&stand->pftlist));
    check(pvec);
    permute(pvec,getnpft(&stand->pftlist));
#endif
    for(p=0;p<getnpft(&stand->pftlist);p++)
      wet[p]=0;
  }
  else
  {
    wet=NULL;
#ifdef PERMUTE
    pvec=NULL;
#endif
  }
  for(l=0;l<LASTLAYER;l++)
    aet_stand[l]=green_transp[l]=0;
  if (config->PM)
    for(l=0;l<LASTLAYER;l++)
      aet_stand[l]=stand->aet_layer[l];
#ifdef PERMUTE
  for(p=0;p<getnpft(&stand->pftlist);p++)
#else
  foreachpft(pft,p,&stand->pftlist)
#endif
  {
#ifdef PERMUTE
    pft=getpft(&stand->pftlist,pvec[p]);
#endif
    /* calculate old or new phenology*/
    if (config->new_phenology)
      phenology_gsi(pft, climate->temp, climate->swdown, day,climate->isdailytemp);
    else
      leaf_phenology(pft,climate->temp,day,climate->isdailytemp);
    cover_stand+=pft->fpc*pft->phen;
    
  /*  if(cutveg(pft) && pft->par->type==GRASS && pft->phen>0.1 && pft->bm_inc.carbon>=20)i{
      isphen=TRUE;
     pft->bm_inc.carbon=5;
pft->phen=0.05;
}
*/
   // *gc_stand=0;
    /* calculate albedo and FAPAR of PFT  already called in update_daily via albedo_stand*/
    albedo_pft(pft, soil->snowheight, soil->snowfraction);
    intercep_stand+=interception(&wet[p],&gi,pft,eeq,climate->prec);
    wet_all+=wet[p]*pft->fpc;
    *gc_stand+=gi;//*pft->fpc;
    stand->gi += gi;//*pft->fpc;
  }
  //if(isphen)
    //harvest_stand(output,stand,0.5);
  /* soil inflow: infiltration and percolation */
  runoff+=infil_perc_rain(stand,climate->prec+melt-intercep_stand,&return_flow_b,withdailyoutput,config);
#ifdef PERMUTE
  for(p=0;p<getnpft(&stand->pftlist);p++)
#else
  foreachpft(pft,p,&stand->pftlist)
#endif
  {
/*
 *  Calculate net assimilation, i.e. gross primary production minus leaf
 *  respiration, including conversion from FPC to grid cell basis.
 *
 */
#ifdef PERMUTE
    pft=getpft(&stand->pftlist,pvec[p]);
#endif
    gpp=water_stressed(pft,&gc,aet_stand,gp_stand,gp_stand_leafon,
                       gp_pft[getpftpar(pft,id)],&gc_pft,&rd,
                       &wet[p],eeq,co2,climate->temp,par,daylength,&wdf,npft,ncft,config);
    *gc_stand+=gc*pft->fpc;//gp_pft[getpftpar(pft,id)];//*pft->fpc;
    //stand->gc+=gc; 
  //*gc_stand+=gc_pft;
    //stand->gp=gp_stand;
    if(gp_pft[getpftpar(pft,id)]>0.0)
    {
      output->gcgp_count[pft->par->id]++;
      output->pft_gcgp[pft->par->id]+=gc_pft/gp_pft[getpftpar(pft,id)];
    }

    npp=npp(pft,gtemp_air,gtemp_soil,gpp-rd,config->with_nitrogen);
    if (withdailyoutput){
      if (output->daily.cft == ALLNATURAL)
        output->daily.npp+=npp;
    }
    output->dcflux-=npp*stand->frac;
    output->mnpp+=npp*stand->frac;
    output->mgpp+=gpp*stand->frac;
    output->mfapar += pft->fapar * stand->frac * (1.0/(1-stand->cell->lakefrac-stand->cell->ml.reservoirfrac));

    output->mphen_tmin += pft->fpc * pft->phen_gsi.tmin * stand->frac * (1.0/(1-stand->cell->lakefrac-stand->cell->ml.reservoirfrac));
    output->mphen_tmax += pft->fpc * pft->phen_gsi.tmax * stand->frac * (1.0/(1-stand->cell->lakefrac-stand->cell->ml.reservoirfrac));
    output->mphen_light += pft->fpc * pft->phen_gsi.light * stand->frac * (1.0/(1-stand->cell->lakefrac-stand->cell->ml.reservoirfrac));
    output->mphen_water += pft->fpc * pft->phen_gsi.wscal * stand->frac * (1.0/(1-stand->cell->lakefrac-stand->cell->ml.reservoirfrac));
    output->mwscal += pft->fpc * pft->wscal * stand->frac * (1.0/(1-stand->cell->lakefrac-stand->cell->ml.reservoirfrac));


    if(pft->stand->type->landusetype==NATURAL)
    {
      if(config->pft_output_scaled)
        output->pft_npp[pft->par->id]+=npp*stand->frac;
      else
        output->pft_npp[pft->par->id]+=npp;
    }
  } /* of foreachpft */

  /* soil outflow: evap and transpiration */
  waterbalance(stand,aet_stand,green_transp,&evap,&evap_blue,wet_all,eeq,cover_stand,
               &frac_g_evap,&ge,FALSE);

  *gc_stand+=ge;//*pft->fpc;
  stand->ge+=ge;//*pft->fpc;
  stand->evap_lastday+=evap+intercep_stand;
  //forrootsoillayer(l){
  //stand->evap_lastday+=aet_stand[l];
//	if (stand->evap_lastday>100||stand->evap_lastday<-100)
//		printf("evap=%g inter=%g aet=%g l=%d\n",evap,intercep_stand,aet_stand[l],l);
//  }
  if(withdailyoutput)
  {
#ifndef COUPLING_WITH_FMS
    // when coupling with FMS/POEM, we always need evap output,
    // regardless whether it is requested in the LPJ output configuration.
    foreachpft(pft,p,&stand->pftlist)
    if(pft->par->id==output->daily.cft)
#endif
      {
        output->daily.evap=evap;
        forrootsoillayer(l)
          output->daily.trans+=aet_stand[l];
        output->daily.irrig=0;
        output->daily.w0=stand->soil.w[1];
        output->daily.w1=stand->soil.w[2];
        output->daily.wevap=stand->soil.w[0];
        output->daily.par=par;
        output->daily.daylength=daylength;
        output->daily.pet=eeq*PRIESTLEY_TAYLOR;
      }
    output->daily.interc=intercep_stand*stand->frac;
    forrootsoillayer(l)
    {
      output->daily.nh4+=stand->soil.NH4[l];
      output->daily.no3+=stand->soil.NO3[l];
      output->daily.nsoil_fast+=stand->soil.pool[l].fast.nitrogen;
      output->daily.nsoil_slow+=stand->soil.pool[l].slow.nitrogen;
    }
  }
  forrootsoillayer(l)
  {
    output->mtransp+=aet_stand[l]*stand->frac;
   // output->dwflux+=aet_stand[l]*stand->frac;//aet_stand[l]*stand->frac;
    output->mtransp_b+=(aet_stand[l]-green_transp[l])*stand->frac;
  }

  output->minterc+=intercep_stand*stand->frac;
  output->mevap+=evap*stand->frac;
  output->dwflux+=(intercep_stand+evap)*stand->frac;  /* for FMS dwflux without evap */
  output->mevap_b+=evap_blue*stand->frac;
  output->mreturn_flow_b+=return_flow_b*stand->frac;
#ifdef COUPLING_WITH_FMS
  stand->cell->exlpj=cover_stand; 
#endif
  if(stand->type->landusetype==NATURAL)
    foreachpft(pft, p, &stand->pftlist)
    {
      output->nv_lai[getpftpar(pft,id)]+=(actual_lai(pft)/NDAYYEAR);
    }

#ifdef DAILY_ESTABLISHMENT
  if (year==911 && day==365) /* TODO: replace the hardcoded value 911 with a more indicative flag like first_year_of_spinup */
    flux_estab=establishmentpft(stand,config->pftpar,npft,config->ntypes,stand->cell->balance.aprec,year);
  else if (year>911)
    flux_estab=establishmentpft(stand,config->pftpar,npft,config->ntypes,stand->cell->balance.aprec,year);
  output->flux_estab.carbon+=flux_estab.carbon*stand->frac;
  output->flux_estab.nitrogen+=flux_estab.nitrogen*stand->frac;
  output->dcflux-=flux_estab.carbon*stand->frac;
#endif

  free(wet);
#ifdef PERMUTE
  free(pvec);
#endif
  return runoff;
} /* of 'daily_natural' */
