/*************************************************************************************/
/**                                                                                \n**/
/**                     g  p  _  s  u  m  .  c                                     \n**/
/**                                                                                \n**/
/**     C implementation of LPJmL                                                  \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**                                                                                \n**/
/**************************************************************************************/

#include "lpj.h"

Real gp_sum(const Stand *stand, /**< Pft list */
            Real co2,              /**< atmospheric CO2 concentration (ppm) */
            Real temp,             /**< temperature (deg C) */
            Real par,              /**< photosynthetic active radiation flux */
            Real daylength,        /**< daylength (h) */
            Real *gp_stand_leafon, /**< pot. canopy conduct.at full leaf cover (mm/s) */
            Real gp_pft[],         /**< pot. canopy conductance for PFTs & CFTs (mm/s) */
            Real *fpc,             /**< total FPC of all PFTs */
            const Config *config   /**< LPJ configuration */
           )
{
  int p;
  Pft *pft;
  Real agd,adtmm,gp,gp_stand,rd,fpc_total,vmax;
  *gp_stand_leafon=gp=*fpc=gp_stand=0;
  if(daylength<1e-20)
  {
    foreachpft(pft,p,&stand->pftlist)
      gp_pft[getpftpar(pft,id)]=0;
    return 0;
  }
  foreachpft(pft,p,&stand->pftlist)
  {
    if isnan(pft->snowcover)
      pft->snowcover=0;
    pft->vmax=0;
    vmax=0;
    if(pft->par->type==CROP)
    {
      adtmm=photosynthesis(&agd,&rd,&vmax,pft->par->path,LAMBDA_OPT,
                           temp_stress(pft->par,temp,daylength),ppm2Pa(co2),
                           temp,
                           par*(1-getpftpar(pft,albedo_leaf))*fpar_crop(pft)*alphaa(pft,config->with_nitrogen,config->laimax_interpolate),
                           daylength);

#ifdef COUPLING_WITH_FMS
      gp=(1.6*adtmm/(ppm2bar(co2)*hour2sec(daylength)))*(1+getpftpar(pft,g1)/sqrt(pft->stand->cell->VPD))+  //pft->stand->cell->VPD
                      pft->par->gmin*fpar_crop(pft);
#else
      
      gp=(1.6*adtmm/(ppm2bar(co2)*(1.0-LAMBDA_OPT)*hour2sec(daylength)))+
                    pft->par->gmin*fpar_crop(pft);
#endif

      gp_stand+=gp;
      gp_pft[getpftpar(pft,id)]=gp;
    }
    else
    {
      adtmm=photosynthesis(&agd,&rd,&vmax,pft->par->path,LAMBDA_OPT,
                           temp_stress(pft->par,temp,daylength),ppm2Pa(co2),
                           temp,
                           par*pft->fpc*alphaa(pft,config->with_nitrogen,config->laimax_interpolate)*(1-getpftpar(pft,albedo_leaf))*(1-pft->snowcover),
                           daylength);
#ifdef COUPLING_WITH_FMS
      gp=(1.6*adtmm/(ppm2bar(co2)*hour2sec(daylength)))*(1+getpftpar(pft,g1)/sqrt(pft->stand->cell->VPD))+  //pft->stand->cell->VPD
                      pft->par->gmin*pft->fpc*(1-pft->snowcover);
#else
      gp=(1.6*adtmm/(ppm2bar(co2)*(1.0-LAMBDA_OPT)*hour2sec(daylength)))+                      
                      pft->par->gmin*pft->fpc*(1-pft->snowcover);
#endif
      gp_pft[getpftpar(pft,id)]=gp*pft->phen;
       
      gp_stand+=gp*pft->phen;//max(pft->phen,0.3);
#ifdef COUPLING_WITH_FMS
    /*  if(pft->par->type==GRASS)
      {
        pft->stand->gpmin=gp*pft->phen/pft->fpc;
        pft->stand->gpmin_leafon=gp/pft->fpc;
      }*/
#endif
    }
    pft->vmax=vmax;
    *fpc+=pft->fpc;
    *gp_stand_leafon+=gp;                                                       // TODO /pft->phen included
  }
#ifdef COUPLING_WITH_FMS
  //gp_stand=max(stand->gpmin,gp_stand);
  //*gp_stand_leafon=max(stand->gpmin_leafon,*gp_stand_leafon);
#endif
  fpc_total=*fpc;
  *gp_stand_leafon= (gp_stand<1e-20 || fpc_total<1e-20) ? 0 : *gp_stand_leafon/fpc_total;
  if isnan(gp_stand)
  {
    printf("gpsumnan: fpc_total=%g, gp=%g  ",fpc_total,gp_stand);
    gp_stand=20;
    foreachpft(pft,p,&stand->pftlist)
      printf("VPD=%g snowcover=%g \n", pft->stand->cell->VPD,pft->snowcover);
  }
  return (gp_stand<1e-20 || fpc_total<1e-20) ? 0 : gp_stand/fpc_total;
} /* of 'gp_sum' */
