/**************************************************************************************/
/**                                                                                \n**/
/**                h  u  m  i  d  i  t  y  .  c                                    \n**/
/**                                                                                \n**/
/**     C implementation of LPJmL                                                  \n**/
/**                                                                                \n**/
/**     Function of of canopy humidity of the individual cell                      \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**                                                                                \n**/
/**************************************************************************************/

#include "lpj.h"

void humidity(Cell *cell,             	     /**< pointer to cell */
              Real temp,         	     /**< canopy temperature (deg C) */
              Dailyclimate climate,   /**< subdaily climate input from FMS */
              Real gc,                	     /**< stomatal conductance for the non PM case (probably not needed in this configuration) */
              Real temp_laststep,     	     /**< temperature of the last timestep (deg C) */
              Real soilwfactor,      	     /**< soilwater factor of all stands combined */
              Real soilw,          	     /**< soil water content of all stands combined */
              Real groundflux_cell,   	     /**< heat flux between canopy temperature and soiltemperature */ 
              Real dt_fast,       	     /**< fast timestep (s) */
              const Config *config   	     /**< LPJmL configuration */
            )
{
#ifdef COUPLING_WITH_FMS
 // Real gc,              /**< waterstressed surface conductance (kg/m2/s) */
  Real vegbeta;         /**< water available for photosynthesis */
  Real vegevap=0;       /**< transpiration for FMS (kg/m2/s) */
  Real delta_t;         /**< temperature difference between actual and last timestep (K) */
  Real delt;            /**< gradient for saturation vapor pressure (kPa) */
  Real VPD;             /**< vapor pressure deficit (kPa) */
  Real rho;             /**< air density (kg/m3) */
  Real qsat1;           /**< saturation specific humidity kg/kg */
  Real psat;            /**< saturation vapor pressure */
  Real veg_c;           /**< canopy conductance to atmospheric boundary layer */
  Real veg_eq, veg_fq;  /**< implicit scheme coefficients for humidity caluclation */
  Real gc_stand=0;        /**< canopy conductance of the whole stand, including ge, gi and gc */
  Real gc_stand_trans=0;   /**< canopy conductance of the whole stand, including gc */
  Real fracsum = 0;     /**< sum of pft fraction in cell */
  Real wet; 		/**< fraction of intercepted water */
  Real PET;             /**< potential evapotranspiration mm/s */
  Real PET_leafon;      /**< potential evapotranspiration for leaf on case mm/s */
  Real swnet;           /**< net solar short wave radiation */
  Real groundflux;      /**< heat flux between canopy temperature and soiltemperature */
  Real rdgas = 287.04;
  Real rvgas = 461.50;
  Real d622  = rdgas/rvgas;
  Real d378  = 1-d622;
  Real d608  = d378/d622;
  /* water stress function variables */
  Real supply_pft,gc_pft,aet,gp_stand,aet_tmp[LASTLAYER],int_store, aet_cor, aet_frac;
  Real rootdist_n[LASTLAYER], aet_layer[LASTLAYER];
  Real supply, demand, wr, roots,root_u,layer,freeze_depth,thaw_depth,root_nu;
  int l,s,p,i;
  Stand *stand;
  Pft *pft;
  Real fpcsum;
  Real supplysum, supply_standsum;
  swnet = climate.swdown*(1-cell->albedo);
  vegbeta = max(0.0, min(1.0, soilwfactor/0.75)); /* calculate water availability for photosynthesis */
  rho = climate.p_surf/(rdgas*(temp+273.15)*(1+ d608*cell->q_ca));
  delta_t = (temp-temp_laststep); /* actual soiltemp - soiltemp last day */
  veg_eq = 1 - climate.dedq / (rho * climate.drag_q); /* implicit scheme coefficient */
  veg_fq = climate.qflux * veg_eq / (rho * climate.drag_q *(1 - veg_eq)); /* implicit scheme coefficient */
  delt = 4098*(0.6108*exp(17.27*temp/(temp+237.3)))/pow((temp+237.3),2);
  VPD = max(0.01,qsat(temp,climate.p_surf)-cell->RH/100*qsat(temp,climate.p_surf)); /* VPD for PM formula */
  cell->VPD2 = max(0.2,VPD); /* VPD for sublimation calculations, factor has to be tuned for stable temperature on ice shields*/
  Real roughness=0;
  supply_standsum=0;
  if (config->PM)
  {
    foreachstand(stand,s,cell->standlist)
    {
      supplysum=0;
      for (l=0;l<LASTLAYER;l++)
        aet_layer[l]=0;
      fpcsum=0;
      foreachpft(pft,p,&stand->pftlist)
        fpcsum += pft->fpc;
      gc_pft=0;
      /* calculation of groundflux for this specific stand */
      groundflux=soilconduct(&stand->soil,0)*(stand->soil.temp2-stand->soil.temp[0])/soildepth[0]*1e3;
      if isnan(stand->gp){
        printf("gpnan");
        stand->gp=0.1;}
      if (stand->gp+stand->ge+stand->gi<0.01){
        stand->gp=0.01;
        stand->ge=0.01;
      }
    //  if (stand->gp+stand->ge+stand->gi<0.1){
    //    stand->ge=0.1-stand->gp-stand->gi;
    //  }

      /* calculation of PET for waterstress calculations and for other processes in LPJ, eg. interception and evaporation in mm/s*/
      PET = max(0,(delt*((swnet+climate.lwnet-groundflux)*0.0864/NSECONDSDAY)+(rho*1.1013e-3*VPD*climate.drag_q))/(delt+0.000665*climate.p_surf/1000*(1+1/((stand->gp*fpcsum+stand->ge+stand->gi)/1000)*climate.drag_q))*NSECONDSDAY/2.45);
      stand->PET += max(0,(delt*((swnet+climate.lwnet-groundflux)*0.0864/NSECONDSDAY)+(rho*1.1013e-3*VPD*climate.drag_q))/(delt+0.000665*climate.p_surf/1000*(1+1/((stand->gp*fpcsum+stand->ge+stand->gi)/1000)*climate.drag_q))*dt_fast/2.45);
      if isnan(stand->PET){
        printf("petnan: VPD=%g, standgp=%g, fpcsum=%g\n",VPD,stand->gp,fpcsum);
        stand->PET=0;}
      PET_leafon = max(0,(delt*((swnet+climate.lwnet-groundflux)*0.0864/NSECONDSDAY)+(rho*1.1013e-3*VPD*climate.drag_q))/(delt+0.000665*climate.p_surf/1000*(1+1/((stand->gp_leafon*fpcsum+stand->ge+stand->gi)/1000)*climate.drag_q))*NSECONDSDAY/2.45);
      foreachpft(pft,p,&stand->pftlist)
      {
        int_store=pft->par->intc*actual_lai(pft);
        if(int_store>0.9999)
          int_store=0.9999;
        /* calculate wet fraction in interception to later correct the demand with it */
        wet=int_store*climate.prec/(PET*dt_fast/NSECONDSDAY);
        wr=layer=root_u=root_nu=aet_cor=0.0;
        aet_frac=1.;
        forrootsoillayer(l)
          rootdist_n[l]=pft->par->rootdist[l];
        /*adjust root layer*/
        if(layerbound[BOTTOMLAYER]>pft->stand->soil.mean_maxthaw && pft->stand->soil.mean_maxthaw>epsilon)
        {
          forrootsoillayer(l)
          {
            layer+=soildepth[l];
            root_u+=pft->par->rootdist[l];
            freeze_depth=layer-pft->stand->soil.mean_maxthaw;
            if (freeze_depth>0)
            {
              thaw_depth=soildepth[l]-freeze_depth;
              rootdist_n[l]=thaw_depth/soildepth[l]*pft->par->rootdist[l];
              root_nu=pft->par->rootdist[l]-rootdist_n[l];
              root_u-= root_nu;
              l++;
              break;
            }
          }
          for(i=l;i<BOTTOMLAYER;i++)
          {
            root_nu+=rootdist_n[i];
            rootdist_n[i]=0;
          }
          for(i=l-1;i>=0;--i)
            rootdist_n[i]=rootdist_n[i]/root_u*root_nu+rootdist_n[i];
        }
        wr=roots=0;
        for(l=0;l<LASTLAYER;l++)
        {
          wr+=rootdist_n[l]*pft->stand->soil.w[l];
          roots+=rootdist_n[l];
        }
        if(wet>0.99)
          wet=0.99;

        if(pft->stand->type->landusetype==AGRICULTURE)
        {
          supply=pft->par->emax*wr*(1-exp(-0.04*((Pftcrop *)pft->data)->ind.root.carbon));
          if (pft->phen>0)
          {
            gp_stand=stand->gp;///pft->phen*fpar(pft);
          }
          else
            gp_stand=stand->gp;
        }
        else
        {
          supply=pft->par->emax*wr*pft->phen;
          gp_stand=stand->gp;
        }
        demand=(1-wet)*PET;
        //pft->wdfcrop+=wdf(pft,demand,supply)*dt_fast/NSECONDSDAY;
        pft->demand+=demand*dt_fast/NSECONDSDAY;
        pft->supply+=supply*dt_fast/NSECONDSDAY;
        if(PET>0 && PET_leafon>0 && pft->fpc>0)
        {
          pft->wscal=(pft->par->emax*wr)/PET_leafon;
          if(pft->wscal>1)
            pft->wscal=1;
        }
        else
          pft->wscal=1;
        pft->wscal_mean+=pft->wscal*dt_fast/NSECONDSDAY;
        aet=(wr>0) ? min(supply,demand)/wr*pft->fpc*dt_fast/NSECONDSDAY : 0;
        if (aet>0 && pft->fpc>epsilon)
        {
          for (l=0;l<LASTLAYER;l++)
          {
            aet_frac=1;
            if(aet*rootdist_n[l]*pft->stand->soil.w[l]/pft->fpc>pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l])
            {
              aet_frac=(pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l])/(aet*rootdist_n[l]*pft->stand->soil.w[l]/pft->fpc);
            }
            aet_tmp[l]=stand->aet_layer[l]+aet*rootdist_n[l]*pft->stand->soil.w[l]*aet_frac;
            if (aet_tmp[l]>pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l])
            {
              aet_cor+=pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l]-stand->aet_layer[l];
              if(aet_cor<epsilon) aet_cor=0;
            }
            else
            {
              aet_cor+=aet*rootdist_n[l]*pft->stand->soil.w[l]*aet_frac;
            }
          }
        }
        else
          aet_cor=0;

        aet=(wr>0) ? aet_cor/wr : 0;
        if (pft->fpc>epsilon && aet>0)
          supply=aet*wr/pft->fpc/dt_fast*NSECONDSDAY;
        else
          supply=0;
        if(supply>=demand)
        {
          gc=gp_stand;///fpcsum;
          pft->gc += gc*dt_fast/NSECONDSDAY;
          gc_pft += gc*pft->fpc;
        }
        else if(PET>0)
        {
          supply_pft = supply *pft->fpc;
          if (supply==0)
            gc=0;
          else  {
	 //   if(pft->stand->type->landusetype==AGRICULTURE)
       // gc= 1000/((delt*((swnet+climate.lwnet-groundflux)*0.0864/NSECONDSDAY)+(rho*1.1013e-3*VPD*climate.drag_q))/(supply/NSECONDSDAY*2.45*(delt+0.000665*climate.p_surf/1000))-1)*climate.drag_q*pft->phen;

         //  else
            gc= 1000/((delt*((swnet+climate.lwnet-groundflux)*0.0864/NSECONDSDAY)+(rho*1.1013e-3*VPD*climate.drag_q))/(supply/NSECONDSDAY*2.45*(delt+0.000665*climate.p_surf/1000))-1)*climate.drag_q*pft->fpc;
            if(pft->par->type==GRASS)
            {
              pft->stand->gpmin=gc/pft->fpc;
            }

         }
          if(gc<0)
            gc=0;
          if (pft->fpc==0)
            pft->gc += 0;
          else
            pft->gc += gc*dt_fast/NSECONDSDAY/pft->fpc;
          gc_pft  += gc;

        }
        else
        {
          gc=0;
          gc_pft+=0;
          pft->gc+=0;
        }
        for (l=0;l<LASTLAYER;l++)
        {
          stand->aet_layer[l]+=aet*rootdist_n[l]*pft->stand->soil.w[l];
          //supplysum+=aet*rootdist_n[l]*pft->stand->soil.w[l]/dt_fast*NSECONDSDAY;
          if (stand->aet_layer[l]>pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l])
            {
            stand->aet_layer[l]=pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l];
            //supplysum=pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l];
            }
        }
        if (demand<0.0001)
          pft->stand->cell->supplyc=1;
        else
          pft->stand->cell->supplyc=supply/demand/0.75;
     /*   if(pft->par->type==GRASS)
        {
          pft->stand->gpmin=gc/pft->fpc;
        }
*/
      supplysum+=aet/dt_fast*NSECONDSDAY;//supply*pft->fpc;
      }
 //     gc_pft=max(stand->gpmin,gc_pft);
      if (isnan(gc_pft)){
        printf("ERROR:NAN_GC");
        gc_pft=0.1;}

      if (fpcsum<0.05)
        gc_pft=1.0;

      
      fracsum += stand->frac;
      gc_stand += (gc_pft+stand->ge+stand->gi)*stand->frac;
      gc_stand_trans += gc_pft*stand->frac;
      supply_standsum += supplysum*stand->frac;
      cell->PET+=PET*dt_fast/NSECONDSDAY*stand->frac;
      roughness+=roughnesslength(cell->standlist)*stand->frac;
    } 
    /* correct PET for lakes TODO: calculate correct evaporation for lakes */
    cell->PET += cell->eeq2*(1-fracsum);
    gc_stand += 10*(1-fracsum); /* use conductance for lakes as 10 mm/day, very rough estimation. TODO: implement PM and conductance calculations for lakes */
    if (fracsum>0)
    {
      gc_stand_trans /= fracsum;
      veg_c=rho / (1/(gc_stand/1000)+(1-vegbeta)/climate.drag_q);
      /* calculation of transpiration, used for FMS. For deserts, gc is set to zero before, so vegevap calculation would produce nan*/
      if (gc_stand_trans<0.01)  
        vegevap=0;
      else
        vegevap = fracsum*((delt*((swnet+climate.lwnet-groundflux_cell)*0.0864/NSECONDSDAY)+(rho*1.1013e-3*VPD*climate.drag_q))/(delt+0.000665*climate.p_surf/1000*(1+1/((gc_stand_trans)/1000)*climate.drag_q))/2.45);
    }
    else
    {
      vegevap=0;
      veg_c=rho / (1/(gc_stand/1000));
    }
    if (stand->soil.snowpack>0.1)
    {
      vegevap=0;
      veg_c=0;//10;//rho / (1/(44/1000)+(1-vegbeta)/climate.drag_q);;
    } 
    //vegevap=supply_standsum/NSECONDSDAY; //for closed waterbalance
    vegevap+=(cell->output.dwflux/NSECONDSDAY+cell->snowflux); /* add evaporation, interception, evaporation from lakes and sublimation */
  }
  else /* if not PM (no subdaily waterstress etc.) - could be used for stand-alone LPJ */
  {
    vegevap = (delt*((swnet+climate.lwnet-groundflux_cell)*0.0864/NSECONDSDAY)+(rho*1.1013e-3*VPD*climate.drag_q))/(delt+0.000665*climate.p_surf/1000*(1+1/((gc)/1000)*climate.drag_q))/2.45;
    veg_c=rho / (1/(gc/1000));
  }

  if isnan(veg_c)
    veg_c=0.00001;
  if isnan(vegevap)
    {
    printf("vegevapnan\n");
    vegevap=0;
    }

  /* evap1 and dedt for temperature calculations */ 
  cell->evap1 = vegevap + veg_c*(climate.qflux - vegevap)/(climate.dedq+veg_c); /* implicit estimate of water vapor flux used for temperature */
  cell->dedt  = delt*veg_c*(1-veg_c/(veg_c+climate.dedq)); /* derivative of evap over T */
  if (isnan(cell->dedt))
    cell->dedt=0;
  if (cell->evap1<0)
    cell->evap1=0;

  cell->q_ca+=(vegevap-climate.qflux+veg_c*delt*delta_t)/(climate.dedq+veg_c); /* canopy height humidity calculation (q_ca (kg/kg))  */
  psat = qsat(temp,climate.p_surf)*1000;
  qsat1 = 0.018*psat/(0.029*(climate.p_surf-psat));
  qsat1 = max(qsat1,0.0001);
  cell->q_ca = max(cell->q_ca,0.0001);
  cell->q_ca = min(cell->q_ca,qsat1);
  if isnan(cell->q_ca)
  {
    cell->q_ca = 0.0001;
    printf("ERROR: QCA NAN");
  }

  /* relative humidity calculation */
  cell->RH = 100 * cell->q_ca / qsat1;
  cell->dRH += cell->q_ca / qsat1 / NSECONDSDAY * dt_fast;
  VPD = psat * (1-cell->RH/100) / 1000; // kPa
  VPD = max(VPD,0.05);
  VPD = min(VPD,2);
  cell->VPD += VPD; /* for gp/gc calculation is divided by number of subdaily timesteps in lpj_climber4.c*/
  /* debug output */
  cell->output.mq_ca += climate.qflux/24*86400;//temp/24;//cell->albedo/24;//vegevap*dt_fast/NSECONDSDAY;//-cell->output.dwflux/NSECONDSDAY*dt_fast/NSECONDSDAY;
#endif
} /* of 'humidity' */
