/**************************************************************************************/
/**                                                                                \n**/
/**             w  a  t  e  r  _  s  t  r  e  s  s  e  d  .  c                     \n**/
/**                                                                                \n**/
/**     C implementation of LPJmL                                                  \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://github.com/PIK-LPJmL/LPJmL                                    \n**/
/**                                                                                \n**/
/**************************************************************************************/

#include "lpj.h"
#include "agriculture.h"

#define EPSILON 0.001  /* min precision of solution in bisection method */

typedef struct
{
  Real fac,co2,temp,apar,daylength,tstress,vmax;
  int path;
} Data;

static Real fcn(Real lambda,Data *data)
{
  Real agd,rd,vmax;

/*
 *              Call photosynthesis to determine alternative total
 *              daytime photosynthesis estimate (adt2) implied by
 *              Eqns 2 & 19, Haxeltine & Prentice 1996, and current
 *              guess for lambda (xmid)
 */
  vmax=data->vmax;
  return data->fac*(1-lambda)-photosynthesis(&agd,&rd,&vmax,data->path,lambda,
                                             data->tstress,data->co2,
                                             data->temp,data->apar,
                                             data->daylength);
/*
 *              Calculate total daytime photosynthesis implied by
 *              canopy conductance from water balance routine and
 *              current guess for lambda (xmid).  Units are mm/m2/day
 *              (mm come from gpd value, mm/day)
 *              Eqn 18, Haxeltine & Prentice 1996
 */

} /* of 'fcn' */

Real water_stressed(Pft *pft, /**< pointer to PFT variables */
                    Real *gc,
                    Real aet_layer[LASTLAYER],
                    Real gp_stand,
                    Real gp_stand_leafon, /**< pot. canopy conduct. at full leaf cover */
                    Real gp_pft, /**< potential canopy conductance */
                    Real *gc_pft,
                    Real *rd,
                    Real *wet,
                    Real eeq,  /**< equilibrium evapotranspiration (mm) */
                    Real co2,  /**< Atmospheric CO2 partial pressure (ppmv) */
                    Real temp, /**< Temperature (deg C) */
                    Real par,  /**< photosynthetic active radiation (J/m2/day) */
                    Real daylength, /**< Daylength (h) */
                    Real *wdf,           /**< water deficit fraction (0..100) */
                    int npft,            /**< number of natural PFTs */
                    int ncft,            /**< number of crop PFTs */
                    const Config *config /**< LPJ configuration */
                   ) /** \return gross primary productivity (gC/m2) */
{
  int l,i,iter;
  Real supply,supply_pft,demand,demand_pft,wr,lambda,gpd,agd,aet,aet_cor,aet_frac;
  Data data;
  Real vmax;
  Real rootdist_n[LASTLAYER];
  Real aet_tmp[LASTLAYER];
  Real layer,root_u,root_nu;
  Real freeze_depth,thaw_depth;
  Real adtmm;
  Real gc_new;
  Real lambda_opt;
  Irrigation *irrig;

  if (config->PM)
  {
    /*adjust root layer*/
//new 5.1 version:
//   getrootdist(rootdist_n,pft->par->rootdist,pft->stand->soil.mean_maxthaw);
/*
//    if(layerbound[BOTTOMLAYER]>pft->stand->soil.mean_maxthaw &&
//        pft->stand->soil.mean_maxthaw>epsilon)
//     {
//       forrootsoillayer(l)
//       {
//         layer+=soildepth[l];
//         root_u+=pft->par->rootdist[l];
//         freeze_depth=layer-pft->stand->soil.mean_maxthaw;
//         if (freeze_depth>0)
//         {
//           thaw_depth=soildepth[l]-freeze_depth;
//           rootdist_n[l]=thaw_depth/soildepth[l]*pft->par->rootdist[l];
//           root_nu=pft->par->rootdist[l]-rootdist_n[l];
//           root_u-= root_nu;
//           l++;
//           break;
//         }
//       }
//       for(i=l;i<BOTTOMLAYER;i++)
//       {
//         root_nu+=rootdist_n[i];
//         rootdist_n[i]=0;
//       }
//       for(i=l-1;i>=0;--i)
//         rootdist_n[i]=rootdist_n[i]/root_u*root_nu+rootdist_n[i];
//     }
*/
//  wr=roots=0;
//  for(l=0;l<LASTLAYER;l++)
// {
//    wr+=rootdist_n[l]*pft->stand->soil.w[l];
//    roots+=rootdist_n[l];
//  }
//    for(l=0;l<BOTTOMLAYER;l++)
//      aet_layer[l]=pft->stand->aet_layer[l];
    *gc=pft->gc;
    gpd=agd=*rd=layer=root_u=root_nu=aet_cor=0.0;
    //*wdf=pft->wdfcrop;
    *wdf=wdf(pft,pft->demand,pft->supply);
  }
  else
  {
    
    gpd=agd=*rd=layer=root_u=root_nu=aet_cor=0.0;
    aet_frac=1.;
    forrootsoillayer(l)
      rootdist_n[l]=pft->par->rootdist[l];
    if(config->permafrost)
    {
      /*adjust root layer*/
      if(layerbound[BOTTOMLAYER]>pft->stand->soil.mean_maxthaw &&
         pft->stand->soil.mean_maxthaw>epsilon)
      {
        forrootsoillayer(l)
        {
          layer+=soildepth[l];
          root_u+=pft->par->rootdist[l];
          freeze_depth=layer-pft->stand->soil.mean_maxthaw;
          if (freeze_depth>0)
          {
            thaw_depth=soildepth[l]-freeze_depth;
            rootdist_n[l]=thaw_depth/soildepth[l]*pft->par->rootdist[l];
            root_nu=pft->par->rootdist[l]-rootdist_n[l];
            root_u-= root_nu;
            l++;
            break;
          }
        }
        for(i=l;i<BOTTOMLAYER;i++)
        {
          root_nu+=rootdist_n[i];
          rootdist_n[i]=0;
        }
        for(i=l-1;i>=0;--i)
          rootdist_n[i]=rootdist_n[i]/root_u*root_nu+rootdist_n[i];
      }
    }
    wr=0;
    for(l=0;l<LASTLAYER;l++)
      wr+=rootdist_n[l]*pft->stand->soil.w[l];

    if(*wet>0.99)
      *wet=0.99;

    if(pft->stand->type->landusetype==AGRICULTURE)
    {
      supply=pft->par->emax*wr*(1-exp(-0.04*((Pftcrop *)pft->data)->ind.root.carbon));
      if (pft->phen>0)
      {
        gp_stand=gp_stand;///pft->phen*fpar(pft);
        gp_pft=gp_pft;///pft->phen*fpar(pft);
      }
    }
    else
    {
      supply=pft->par->emax*wr*pft->phen;
    }

    supply_pft=supply*pft->fpc;
    demand=(gp_stand>0) ? (1.0-*wet)*eeq*param.ALPHAM/(1+(param.GM*param.ALPHAM)/gp_stand) : 0;
    demand_pft=(gp_pft>0) ? (1.0-*wet)*eeq*param.ALPHAM/(1+(param.GM*param.ALPHAM)/gp_pft) : 0;
    *wdf=wdf(pft,demand,supply);

    if(eeq>0 && gp_stand_leafon>0 && pft->fpc>0)
    {
      pft->wscal=(pft->par->emax*wr)/(eeq*param.ALPHAM/(1+(param.GM*param.ALPHAM)/gp_stand_leafon));
      if(pft->wscal>1)
        pft->wscal=1;
    }
    else
      pft->wscal=1;

    pft->wscal_mean+=pft->wscal;

    if(supply_pft>=demand_pft)
      *gc_pft=gp_pft;
    else if(eeq>0)
    {
      *gc_pft=(param.GM*param.ALPHAM)*supply_pft/((1.0-*wet)*eeq*param.ALPHAM-supply_pft);
      if(*gc_pft<0)
        *gc_pft=0;
    }
    else
      *gc_pft=0;

    aet=(wr>0) ? min(supply,demand)/wr*pft->fpc : 0;
    if (aet>0 && pft->fpc>epsilon)
    {
      for (l=0;l<LASTLAYER;l++)
      {
        aet_frac=1;
        if(aet*rootdist_n[l]*pft->stand->soil.w[l]/pft->fpc>pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l])
        {
          aet_frac=(pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l])/(aet*rootdist_n[l]*pft->stand->soil.w[l]/pft->fpc);
        }
        aet_tmp[l]=aet_layer[l]+aet*rootdist_n[l]*pft->stand->soil.w[l]*aet_frac;
        if (aet_tmp[l]>pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l])
        {
          aet_cor+=pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l]-aet_layer[l];
          if(aet_cor<epsilon) aet_cor=0;
        }
        else
        {
          aet_cor+=aet*rootdist_n[l]*pft->stand->soil.w[l]*aet_frac;
        }
      }
    }
    else
      aet_cor=0;

    aet=(wr>0) ? aet_cor/wr : 0;
    if (pft->fpc>epsilon && aet>0)
      supply=aet*wr/pft->fpc;
    else
      supply=0;
    if(supply>=demand)
    {
  //  temp=min(temp,25);
      *gc=gp_stand;
    }
    else if(eeq>0)
    {
      *gc=(param.GM*param.ALPHAM)*supply/((1.0-*wet)*eeq*param.ALPHAM-supply);
      if(*gc<0)
        *gc=0;
    }
    else
      *gc=0;
  } /* of if(config->PM) */
  if(pft->par->type==CROP)
    gpd=NSECONDSDAY*(*gc*pft->fpc-pft->par->gmin*fpar(pft));
//    gpd=hour2sec(daylength)*(*gc-pft->par->gmin*fpar(pft));
  else
  //  gpd=NSECONDSDAY*(*gc-pft->par->gmin*pft->phen)*pft->fpc;
    gpd=NSECONDSDAY*(*gc*pft->fpc-pft->par->gmin*fpar(pft));
  data.tstress=temp_stress(pft->par,temp,daylength);
  if(gpd>1e-5 && isphoto(data.tstress)) //1e-5
  {
    data.fac=gpd/1.6*ppm2bar(co2);
    data.path=pft->par->path;
    data.temp=temp;
    data.co2=ppm2Pa(co2);
    data.apar=par*(1-getpftpar(pft, albedo_leaf))*alphaa(pft,config->with_nitrogen,config->laimax_interpolate)*fpar(pft); /** par calculation do not include albedo*/
    data.daylength=daylength;
    data.vmax=pft->vmax;
#ifdef COUPLING_WITH_FMS
    lambda_opt=1-1./(1+getpftpar(pft,g1)/sqrt(pft->stand->cell->VPD));
#else
    lambda_opt=LAMBDA_OPT;
#endif
    lambda=bisect((Bisectfcn)fcn,0.02,lambda_opt+0.05,&data,0,EPSILON,30,&iter);
    vmax=pft->vmax;
    //lambda=lambda_opt;  werner
    //vmax=0;  werner
    adtmm=photosynthesis(&agd,rd,&vmax,data.path,lambda,data.tstress,data.co2,
    //adtmm=photosynthesis(&agd,rd,&vmax,data.path,LAMBDA_OPT,data.tstress,data.co2,   werner
                   temp,data.apar,daylength);
    gc_new=(1.6*adtmm/(ppm2bar(co2)*(1.0-lambda)*hour2sec(daylength)))+
                    pft->par->gmin*fpar(pft);
    if(*gc>gc_new)
      *gc=gc_new;
    pft->vmax=vmax;
    if(config->with_nitrogen)
    {
      nitrogen_stress(pft,temp,daylength,npft,config->nbiomass,ncft,config->permafrost);

      adtmm=photosynthesis(&agd,rd,&pft->vmax,data.path,lambda,data.tstress,data.co2,
                     temp,data.apar,daylength);
      *gc=(1.6*adtmm/(ppm2bar(co2)*(1.0-lambda)*hour2sec(daylength)))+
                    pft->par->gmin*fpar(pft);
      demand=(*gc>0) ? (1-*wet)*eeq*param.ALPHAM/(1+(param.GM*param.ALPHAM)/(*gc)) :0;
      if(gc_new-*gc>0.01 &&  demand-supply_pft>0.1)
      {
         *gc=(param.GM*param.ALPHAM)*supply_pft/((1.0-*wet)*eeq*param.ALPHAM-supply_pft);
         if(*gc<0)
           *gc=0;
         if(pft->stand->type->landusetype==AGRICULTURE)
           gpd=hour2sec(daylength)*(*gc-pft->par->gmin*fpar(pft));
         else
           gpd=hour2sec(daylength)*(*gc-pft->par->gmin*pft->phen*pft->fpc*(1-pft->snowcover));
        data.fac=gpd/1.6*ppm2bar(co2);
        data.vmax=pft->vmax;
        lambda=bisect((Bisectfcn)fcn,0.02,lambda,&data,0,EPSILON,20,&iter);
        adtmm=photosynthesis(&agd,rd,&pft->vmax,data.path,lambda,data.tstress,data.co2,
                             temp,data.apar,daylength);
        *gc=(1.6*adtmm/(ppm2bar(co2)*(1.0-lambda)*hour2sec(daylength)))+
                      pft->par->gmin*fpar(pft);
        demand=(*gc>0) ? (1-*wet)*eeq*param.ALPHAM/(1+(param.GM*param.ALPHAM)/(*gc)) :0;
      }
      aet=(wr>0) ? demand*fpar(pft)/wr :0 ;

      if(vmax>0)
        pft->nlimit+=pft->vmax/vmax;
      if(pft->stand->type->landusetype==AGRICULTURE){
        irrig=pft->stand->data;
        if(&pft->stand->cell->output.daily!=NULL &&
          pft->par->id==pft->stand->cell->output.daily.cft &&
          irrig->irrigation==pft->stand->cell->output.daily.irrigation){
            pft->stand->cell->output.daily.nlimit=pft->vmax/vmax;
        }
      }
    }
    /* in rare occasions, agd(=GPP) can be negative, but shouldn't */
    agd=max(0,agd);
    *rd=*rd;    /* DON'T DELETE THIS LINE */
  }
  else
    agd=0;
  if (!config->PM)
  {
    for (l=0;l<LASTLAYER;l++)
    {
      aet_layer[l]+=aet*rootdist_n[l]*pft->stand->soil.w[l];
      if (aet_layer[l]>pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l])
        aet_layer[l]=pft->stand->soil.w[l]*pft->stand->soil.par->whcs[l];
    }
    if (demand<0.0001)
       pft->stand->cell->supplyc=1;
    else
       pft->stand->cell->supplyc=supply/demand/0.75;
  }
  return agd;
} /* of 'water_stressed' */
