/**************************************************************************************/
/**                                                                                \n**/
/**                     l p j _ c l i m b e r 4  .  c                              \n**/
/**                                                                                \n**/
/**     coupling LPJmL4 to CLIMBER-4                                               \n**/
/**                                                                                \n**/
/** (C) Potsdam Institute for Climate Impact Research (PIK), see COPYRIGHT file    \n**/
/** authors, and contributors see AUTHORS file                                     \n**/
/** This file is part of LPJmL and licensed under GNU AGPL Version 3               \n**/
/** or later. See LICENSE file or go to http://www.gnu.org/licenses/               \n**/
/** Contact: https://gitlab.pik-potsdam.de/lpjml                                   \n**/
/**                                                                                \n**/
/**************************************************************************************/

/**
 * \file
 * C-side of the wrapper for coupling LPJ to Climber-4 / land_lad.
 * This wrapper replaces the LPJ main program when LPJ is
 * compiled as part of Climber-4.
 * It is not compiled for stand-alone LPJ.
 * See lpjml.c for information about the original main program.
 */

/**
 * \file
 * This block contains those functions that are called by the F90-implemented land_lad
 * resp. land_atlantes.
 * It is tested with our two main target platforms GNU gfortran with g++ and Intel Fortran
 * with Intel C++, both on Linux.
 * For other Fortran/C++ combos, different naming and argument passing conventions might
 * be needed.
 * GNU and Intel Fortran convert names of external functions to lowercase and append one
 * underscore. (This is their default behaviour, can be changed with command line options and/or
 * special directive comments in the code.)
 */

#include <time.h>
#include <math.h>
#include <assert.h>
#include "lpj.h"
#include "grass.h"
#include "tree.h"
#include "crop.h"
#include "natural.h"
#include "grassland.h"
#include "biomass_tree.h"
#include "biomass_grass.h"
#include "agriculture.h"

#include "cpl.h"

//#define TEST_CPL

//#ifndef DAILY_ESTABLISHMENT
//#error for coupling with FMS/MOM4/Climber-4 , DAILY_ESTABLISHMENT must be #defined
//#endif

#define NTYPES 3 /*< number of plant functional types: grass, tree, crop, bioenergy */
#define NSTANDTYPES 9 /*< number of stand types / land use types as defined in landuse.h */

#define dflt_conf_filename_ml "lpjml_fms.js" /*< Default LPJ configuration file
                                              if called by lpjml */
#define dflt_conf_filename "lpj.conf" /*< Default LPJ configuration file
                                         if called by lpj */

static const char *progname;

static Outputfile *output; /*< Output file array */
static Input input;        /*< input data */
static Config config;      /*< LPJ configuration */
static Cell *grid;         /*< cell array */
static Standtype standtype[NSTANDTYPES];

static int npft;           /*< Number of natural PFT's */
static int ncft;           /*< Number of crop PFT's */

/// from iterate()
static Real co2,cflux_total;
static Flux flux;
static int year, landuse_year, wateruse_year;
#ifdef STORECLIMATE
static Climatedata store,data_save;
#endif

static double glon_min, glon_max, glat_min, glat_max;


static int is, ie, js, je;    ///< boundary indices of local compute domain
static MPI_Comm comm;         ///< MPI communicator for the atmosphere domain
static int pe, root_pe, npes; ///< our own MPI rank, root MPI rank, total number of atmosphere ranks

static Cpl *cpl;              ///< mapping between LPJ grid cell list and FMS rectangular domain decomposition

static int dt_fast; /*< fast time step [sec] */

///TODO: adapt the mapping for domain decomposition.
///  Major problem: FMS and LPJ use different decomposition schemes
///
/// Information for mapping between FMS compute domain lat-lon-arrays
/// and LPJ grid cell indices.
/// We assume that both FMS and LPJ have longitudes running 0..360 degrees (not radians, not -180..180)
static double lpj_lon_min = 100000, lpj_lon_max = -100000, lpj_lat_min = 100000, lpj_lat_max = -100000; //< cell center coordinates of bounding box
static int nlons, nlats; //< number of cells in lon/lat directions of the bounding box in the FMS domain
//static int *lpj_to_fms_indices = NULL; //< array to map LPJ cell index to FMS array index, on local compute domain
//static int *fms_to_lpj_indices = NULL; //< array to map FMS array index to LPJ cell index


//// evaporation / transpiration is accumulated for monthly sums in the cell output structure.
//// daily values are available only in local variables of some functions.
//// Thus, we remember here yesterday's monthly sums, and the difference to today's values
//// is the daily amount. The XXX_yesterday values must be reset to 0 every month.
//static Real *mevap_yesterday = NULL, /* from bare soil */
//  *mtransp_yesterday = NULL, /* from transpiration of the plants */
//  *minterc_yesterday = NULL; /* should be included in the transpiration??? */
//// mevap_lake and mevap_res are computed separately in update_daily.c ,
//// thus we must treat those also here, and finally sum up all four evap components for passing to the coupler.
//static Real *mevap_lake_yesterday = NULL, /* from lakes */
//  *mevap_res_yesterday = NULL; /* evap from reservoires, if in use */

#ifdef USE_FMSCLOCK
// FMS clock id to measure time for the data exchange in the coupling between
// LPJ and FMS domain decomposition
static int lpj_update_cpl_clock_id = -1;
int lpj_laketemp_clock_id = -1; // FMS clock id to measure time spent in laketemp()
// fortran functions in fmsclock.F90
extern int fmsclock_id_(const char *name, int namelen); // sigh. passing strings from C to Fortran is not tested
inline int fmsclock_id(const char *name) { return fmsclock_id_(name, strlen(name)); }
extern void fmsclock_begin_(const int *id);
extern void fmsclock_end_(const int *id);
#endif

/// Compare double values for equality.
/// They are considered equal if the relative error is smaller than epsilon.
inline static Bool double_eq(const double v1, const double v2, const double e) {
  return fabs((v1-v2)/((v1+v2)/2)) < e;
}


/**
 * lpj_init_() first of all is used to check for
 * compatibility of the procedure call interface between
 * the Fortran90 and C++ modules.
 * Possible pitfalls include:
 * - size of Fortran integer vs. sizeof C++ int
 * - size of Fortran real vs. sizeof C++ float or double
 * - assumed size arrays and pointers might be passed as
 *   descriptors (alias dope vectors)
 * - alignment of elements in derived types
 *   => to avoid this problem, we try to avoid passing
 *      derived types, and use the single elements as call
 *      arguments (we shall see if this will always be possible)
 *
 * TODO: check data types for char * parameters from Fortran
 * However, non are used here yet.
 */
#ifdef __cplusplus
extern "C"
#endif
void lpj_init_
(
 /*! first some arguments for checking the F90->C++ interface */
 const int *const deadbeef, const double *const pitest,
 const int *const true_in, const int *const false_in,
 const double *const testarray3d,
 const double *const testarrayco2, // allocate()d 2D test array
 const int *const kind_i, const int *const kind_r, const int *const kind_l,

 /*! now the ``real'' arguments */
 /*! the FMS time stamps are broken out into days and seconds */
 const int *const Time_init_year, const int *const Time_init_month, const int *const Time_init_day,
 const int *const Time_init_hour, const int *const Time_init_minute, const int *const Time_init_second,
 const int *const Time_year, const int *const Time_month, const int *const Time_day,
 const int *const Time_hour, const int *const Time_minute, const int *const Time_second,
 const int *const Time_step_fast /* seconds */, const int *const Time_step_slow /* seconds */,

 /*! information about domain partitioning */
 const int *const peset_in,   /*< MPI communicator for the set of atmoshpere pes */
 const int *const pe_in,      /*< our own MPI rank == processing element number */
 const int *const root_pe_in, /*< MPI rank of atmosphere master process */
 const int *const npes_in,    /*< number of total atmosphere processes */
 const int *const layout,     /*< 2-element array with numberof PEs in lon and lat directions */
 
 /*! cell centers of corners of global grid */
 const double *const glon_min_in,
 const double *const glon_max_in,
 const double *const glat_min_in,
 const double *const glat_max_in,

 /*! the boundary indices of the local compute domain */
 const int *const is_in, const int *const ie_in, const int *const js_in, const int *const je_in,

 /*! grid axis info */
 const int *const nlons_in, /*< number of cells in the FMS domain at land_lad resolution in lon/lat directions */
 const int *const nlats_in, /*< just used for testing and curiosity */
 const double *const minlonb_in, const double *const maxlonb_in,
 const double *const minlatb_in, const double *const maxlatb_in, /*< bounding box of FMS compute domain */

 const int *const lpj_update_cpl_clock_id_in,

 /*! LPJ-specific options TODO */
 const int *const argc_in, /* char **argv,*/

 /*! Output parameters */
 int *global_number_lpj_cells, /* = config.nall */

 /*! and finally some interface-checking args again */
   const double *const onetwothree, const int *const fse
 ) { /* begin of lpj_init_() */

  const double myonetwothree = 1234567809.0987654321L;

  int argc = *argc_in;
  char **argv = NULL;

  int i, j, k,cell;

  printf("%s %s F90->C interface test\n", __FILE__, __FUNCTION__); fflush(stdout);
  printf("Fortran KIND(integer) %d KIND(real) %d KIND(logical) %d C++ sizeof int %d long %d float %d double %d Bool %d\n",
         *kind_i, *kind_r, *kind_l, (int)sizeof(int), (int)sizeof(long), (int)sizeof(float), (int)sizeof(double),
         (int)sizeof(Bool));
  fflush(stdout);
  if (*kind_i != sizeof(int)) { printf("Error: Fortran KIND(integer) != C++ sizeof(int)\n"); fflush(stdout); abort(); }
  if (*kind_r != sizeof(double)) { printf("Error: Fortran KIND(real) != C++ sizeof(double)\n"); fflush(stdout); abort(); }
  if (*kind_l != sizeof(int)) { printf("Error: Fortran KIND(logical) != C++ sizeof(int)\n"); fflush(stdout); abort(); }
  if (*deadbeef != 0xeadbeef /* 0xdeadbeef yields overflow error on 32bit machines */) {
    printf("Error: test1 failed: 0xdeadbeef != %x\n", *deadbeef);
    fflush(stdout);
    abort();
  }
  if (! double_eq(*pitest, M_PI, epsilon)) {
    printf("Error: test2 failed: %30.28f != %30.28f\n", M_PI, *pitest); fflush(stdout);
    abort();
  }
  if (! (*true_in)) { printf("Error: test3 failed: true_in %x is not true\n", *true_in);
    fflush(stdout);
    abort();
  }
  if (*false_in) { printf("Error: test4 failed: false_in %x is not false\n", *false_in);
    fflush(stdout);
    abort();
  }

  for (k = 1; k <= 4; k++) {
    for (j = 1; j <= 3; j++) {
      for (i = 1; i <= 2; i++) {
        const int idx = (k-1)*3*2 + (j-1)*2 + (i-1);
        const double testval = testarray3d[idx];
        const double myval = i*100.0+j+k/10.0;
        /*printf("testarray3d(%2d,%2d,%2d) = testarray3d[%2d] = %10.9g vs. %10.9g\n",
         *     i, j, k, idx, testval, myval);
         */
        if (! double_eq(testval, myval, epsilon)) {
          printf("Error: test5 failed: testarray3d(%d,%d,%d) is %10.9g != %10.9g\n", i, j, k, testval, myval);
          fflush(stdout);
          abort();
        }
      }
    }
  }

  if (! double_eq(*onetwothree, myonetwothree, epsilon)) {
    printf("Error: test6 failed: %24.12f != %24.12f\n", myonetwothree, *onetwothree);
    fflush(stdout);
    abort();
  }
  if (*fse != 4711) { printf("Error: test7 failed: 4711 != %d\n", *fse); fflush(stdout); abort(); }
  /*printf("double HUGE_VAL is %g\n", HUGE_VAL);*/

  /* First interface tests finished,
   * now look at the actual model parameters
   */
  is = *is_in; ie = *ie_in; js = *js_in; je = *je_in;

  /* and with these we can do some more interface tests */
  //for (j = js; j <= je; j++) {
  //  for (i = is; i <= ie; i++) {
  //    int index = (i-is)+(j-js)*(ie-is+1);
  //    printf("%3d %3d %5d %f\n", i, j, index, testarrayco2[index]);
  //  }
  //}
  if (! double_eq(testarrayco2[(is-is) + (js-js)*(ie-is+1)], 42.42, epsilon)) {
    printf("Error: 2-D test array corner [%d,%d] %f != 42.42\n", is-is, js-js, testarrayco2[(is-is) + (js-js)*(ie-is+1)]);
    fflush(stdout); abort();
  }
  if (! double_eq(testarrayco2[(ie-is) + (js-js)*(ie-is+1)], 43.43, epsilon)) {
    printf("Error: 2-D test array corner [%d,%d] %f != 43.43\n", ie-is, js-js, testarrayco2[(ie-is) + (js-js)*(ie-is+1)]);
    fflush(stdout); abort();
  }
  if (! double_eq(testarrayco2[(is-is) + (je-js)*(ie-is+1)], 44.44, epsilon)) {
    printf("Error: 2-D test array corner [%d,%d] %f != 44.44\n", is-is, je-js, testarrayco2[(is-is) + (je-js)*(ie-is+1)]);
    fflush(stdout); abort();
  }
  if (! double_eq(testarrayco2[(ie-is) + (je-js)*(ie-is+1)], 45.45, epsilon)) {
    printf("Error: 2-D test array corner [%d,%d] %f != 45.45\n", ie-is, je-js, testarrayco2[(ie-is) + (je-js)*(ie-is+1)]);
    fflush(stdout); abort();
  }

  printf("... interface test finished\n"); fflush(stdout);

  pe = *pe_in; root_pe = *root_pe_in; npes = *npes_in;
  comm = *peset_in;
  printf("MPI: comm %d=0x%x pe %d root_pe %d npes %d layout %d x %d\n", comm, comm, pe, root_pe, npes, layout[0], layout[1]);
  glon_min = *glon_min_in; glon_max = *glon_max_in; glat_min = *glat_min_in; glat_max = *glat_max_in;
  printf("cell centers of corners of global grid at %g/%g %g/%g\n", glon_min, glat_min, glon_max, glat_max);
  printf("Domain: local boundary indices: is %d ie %d js %d je %d\n", is, ie, js, je);

  if (pe == root_pe) {
    printf("Time_init: %04d-%02d-%02d %02d:%02d:%02d\n",
           *Time_init_year, *Time_init_month, *Time_init_day,
           *Time_init_hour, *Time_init_minute, *Time_init_second);
    printf("Time now : %04d-%02d-%02d %02d:%02d:%02d\n",
           *Time_year, *Time_month, *Time_day,
           *Time_hour, *Time_minute, *Time_second);
    printf("Time step fast: %d seconds\n", *Time_step_fast);
    printf("Time step slow: %d seconds\n", *Time_step_slow);
  }
  dt_fast = *Time_step_fast;

  /* fake the argument vector */
  argv = (char **)malloc((argc+1) * sizeof(*argv));
  if (NULL == argv) { perror("cannot malloc argv for LPJ"); exit(1); }
  argv[argc] = NULL;
  //fprintf(stderr, "Here pointercheck should report an error %d\n", *(int *)(argv[argc+1]));
  argv[0] = strdup("lpjml"); /* or lpjml ? or something else ? */
  if (argc > 1) {
    /* TODO: fill in the other entries */
    for (i = 1; i < argc; i++) argv[i] = NULL;
  }

  lpj_update_cpl_clock_id = *lpj_update_cpl_clock_id_in;

  /*! LPJ must begin all its runs at start of a fresh year ! */
  /*! TODO: check that here. */
  /* Also: LPJ reads its start spinup, start, and count-of-to-be-simulated years from
   * its configuration file, inside readconfig() below
   * (actually it happens in fscanconfig() section V.)
   * When coupling with  FMS, we want to use this information here that was passed from
   * FMS instead of the numbers from the lpj config file.
   * On the other hand, it might be useful to check if both dates match.
   */

  int rc;

//#ifndef DAILY_ESTABLISHMENT
//  fprintf(stderr, "Error: for coupling into POEM, LPJ must be compiled with flag -DDAILY_ESTABLISHMENT\n");
//  fflush(stderr);
//  abort();
//#endif

  /* Create array of functions, uses the typedef of (*Fscanpftparfcn) in pft.h */
  Fscanpftparfcn scanfcn[NTYPES]={fscanpft_grass,fscanpft_tree,fscanpft_crop};

  standtype[NATURAL]=natural_stand;
  standtype[SETASIDE_RF]=setaside_rf_stand;
  standtype[SETASIDE_IR]=setaside_ir_stand;
  standtype[AGRICULTURE]=agriculture_stand;
  standtype[MANAGEDFOREST]=managedforest_stand;
  standtype[GRASSLAND]=grassland_stand,
  standtype[BIOMASS_TREE]=biomass_tree_stand,
  standtype[BIOMASS_GRASS]=biomass_grass_stand,
  standtype[KILL]=kill_stand;

  enablefpe();
  /*
   * Use default communicator containing all processors. In defining your own
   * communicator it is possible to run LPJ on a subset of processors
   * HERE: use the communicator from FMS/MPP domain
   */
  initmpiconfig(&config,comm);

  /* TODO: use parameters passed from F90 part instead of argv */
  progname=strippath(argv[0]); /* strip path from program name */
  if(argc>1)
  {
    if(!strcmp(argv[1],"-h")) /* check for help option */
    {
      if(isroot(config))
      {
        help(progname,
             (strcmp(progname,"lpj")) ? dflt_conf_filename_ml : dflt_conf_filename);
      }
      MPI_Finalize();
      /*return EXIT_SUCCESS*/ exit(0);
    }
    else if(!strcmp(argv[1],"-v")) /* check for version option */
    {
      if(isroot(config))
        printflags(progname);
      MPI_Finalize();
      /*return EXIT_SUCCESS*/ exit(0);
    }
  }
  if(isroot(config))
  { /* Output only for the root task 0 */
    time_t tbegin;
    time(&tbegin);
    copyright(progname);
    printf("\nRunning for user %s on %s at %s",getuser(),gethost(),
           ctime(&tbegin));
    fflush(stdout);
  }

  /* NTYPES is set to natural vegetation must be considered
   * in light and establishment
   * crops must have last id-number */
  /* Read configuration file */
  rc=readconfig(&config,
                (strcmp(progname,"lpj")) ? dflt_conf_filename_ml :
                                           dflt_conf_filename,
                scanfcn,NTYPES,NOUT,&argc,&argv,lpj_usage);
  failonerror(&config,rc,READ_CONFIG_ERR,"Cannot read configuration");
  if(isroot(config) && argc)
    fprintf(stderr,"WARNING018: Arguments listed after configuration filename, will be ignored.\n");

  /*! check that the date passed from FMS matches the date from the LPJ configuration */
  /* alternate TODO: set the date from FMS to the LPJ configuration structure */
  if(isroot(config))
    printf("Current FMS date: %04d-%02d-%02d. Current LPJ year: %04d\n",
           *Time_year, *Time_month, *Time_day, config.firstyear);
  if (isroot(config) && 1 != *Time_month || 1 != *Time_day) {
    printf("Warning: FMS start time is not at begin of new year\n");
    /*fflush(stdout); abort();*/
  }
  if (isroot(config) && *Time_year != config.firstyear) {
    printf("Warning: FMS start year differs from LPJ start year\n");
    /*fflush(stdout); abort();*/
  }

  /* Check that river routing was enabled in the configuration
   * The original LPJ can also work (faster) wthout river routing,
   * but for coupling with FMS we always want it enabled.
   */

#ifdef IMAGE
  if(config.sim_id==LPJML_IMAGE)
  {
    rc=open_image(&config);
    failonerror(&config,rc,OPEN_IMAGE_ERR,"Cannot open IMAGE coupler");
  }
#endif
  if(isroot(config))
    printconfig(&config,config.npft[GRASS]+config.npft[TREE],
                config.npft[CROP]);

  rc=((grid=newgrid(&config,standtype,NSTANDTYPES,config.npft[GRASS]+config.npft[TREE],config.npft[CROP]))==NULL);
  failonerror(&config,rc,INIT_GRID_ERR,"Initialization of LPJ grid failed");
  rc=initinput(&input,grid,config.npft[GRASS]+config.npft[TREE],config.npft[CROP],&config);
  failonerror(&config,rc,INIT_INPUT_ERR,
              "Initialization of input data failed");

  /* open output files */
  output=fopenoutput(grid,NOUT,&config);

  if(isopen(output,GRID))
    writecoords(output,GRID,grid,&config);
  if(isopen(output,COUNTRY) && config.withlanduse)
    writecountrycode(output,COUNTRY,grid,&config);
  if(isopen(output,REGION) && config.withlanduse)
    writeregioncode(output,REGION,grid,&config);
  if(config.with_anomaly)
  {
  if(getclimate(input.climate,grid,config.firstyear,&config))
  {
    fprintf(stderr,"ERROR104: Simulation stopped in getclimate().\n");
    fflush(stderr);
    /*break;*/ /* leave time loop */ abort();
  }
    for(cell=0;cell<config.ngridcell;cell++)
      grid[cell].aprec_poem_last=grid[cell].aprec_ref=getaprec(input.climate,cell);
  }
  if(isroot(config))
    printf("Simulation begins...\n");

  //year=iterate(output,grid,input,
  //             config.npft[GRASS]+config.npft[TREE],config.npft[CROP],
  //             &config);

  //int iterate(Outputfile *output, /* Output file data */
  //            Cell grid[],        /* cell grid array */
  //            Input input,        /* input data: climate, land use, water use */
  //            int npft,           /* Number of natural PFT's */
  //            int ncft,           /* Number of crop PFT's */
  //            Config *config   /* grid configuration data */
  //            )                    /* returns last year+1 on success */

  npft = config.npft[GRASS]+config.npft[TREE];
  ncft = config.npft[CROP];

  // from iterate()
  //Bool rc;
#ifdef STORECLIMATE
  if(config.nspinup)
  {
    //fprintf(stderr, "Error: The current LPJ configuration specifies %d spinup years, but when coupled to Climber-4, no Spinup is possible\n",
    //        config.nspinup);
    //fflush(stderr);
    //abort();

    /* climate for the first nspinyear years is stored in memory
       to avoid reading repeatedly from disk */
    rc = storeclimate(&store,input.climate,grid,input.climate->firstyear,config.nspinyear,&config);
    failonerror(&config,rc,STORE_CLIMATE_ERR,"Storage of climate failed");

    data_save=input.climate->data;
  }
#endif
  if(config.initsoiltemp)
    initsoiltemp(input.climate,grid,&config);


  /* main loop over spinup + simulation years  */
  // for(year=config.firstyear-config.nspinup;year<=config.lastyear;year++)

  /*
   * --- end of the code that was copied from lpjml.c ---
   */

  //lpj_update_cpl_clock_id = fmsclock_id("LPJ_update_cpl"); already done in lpj_driver.F90 to pull in the fmsclock.o object file
  lpj_laketemp_clock_id = fmsclock_id("LPJ_update_laketemp");

  /**
   * Create a mapping from Fortran90-lat-lon arrays to LPJ cell indices.
   * The mapping is needed for storing input data from land_lad/FMS into LPJ,
   * and to extract LPJ results vice versa.
   * When running in parallel, the big problem is that LPJ uses a different
   * domain decomposition scheme than FMS.
   * FMS just uses a 2-D rectangular decompistion of the global rectilinear grid.
   * The LPJ grid is not global, but contains only the catual land cells,
   * and LPJ tries to minimize inter-domain communication along river routes.
   * The mapping functions are intended to work on LPJ's own grid resolution.
   * Interpolation from/to possibly different grid in land_lad is handled on the
   * Fortran side.
   *
   * Sigh. The official LPJ input data set uses longitudes -180 .. 180, while land_lad uses
   * 0 .. 360 .
   *
   * From src/netcdf/createcoord_all.c
   * This code creates a bounding box around all land cells.
   * But when coupled to FMS, also ocean cells are contained
   * in the FMS domain. Thus we must embed the LPJ cells in the FMS domain.
   * <PRE>
   *   array->lon_min=array->lat_min=1000;
   *   lon_max=lat_max=-1000;
   *   for(cell=0;cell<config->nall;cell++)
   *   {
   *     if(array->lon_min>lon[cell])
   *       array->lon_min=lon[cell];
   *     if(lon_max<lon[cell])
   *       lon_max=lon[cell];
   *     if(array->lat_min>lat[cell])
   *       array->lat_min=lat[cell];
   *     if(lat_max<lat[cell])
   *       lat_max=lat[cell];
   *   }
   *   array->nlon=(int)((lon_max-array->lon_min)/config->resolution.lon)+1;
   *   array->nlat=(int)((lat_max-array->lat_min)/config->resolution.lat)+1;
   *   for(cell=0;cell<config->nall;cell++)
   *   {
   *     array->index[cell]=(int)((lon[cell]-array->lon_min)/config->resolution.lon+0.5)+
   *                      (int)((lat[cell]-array->lat_min)/config->resolution.lat+0.5)*array->nlon;
   * #ifdef SAFE
   *     if(array->index[cell]<0 || array->index[cell]>=array->nlon*array->nlat)
   *       fprintf(stderr,"Invalid index %d.\n",array->index[cell]);
   * #endif
   *   }
   * </PRE>
   *
   * This code here was tested using one single CPU.
   * TODO: It is intended to also run in parallel, but that still needs to be tested.
   */
  // Obtaining LPJ cell boundary box is just for testing.
  {
    int lpj_nlons, lpj_nlats;
    double minlonb = 1000, maxlonb = -1000, minlatb = 1000, maxlatb = -1000;
    for (i = 0; i < config.ngridcell; i++) {
      if (lpj_lon_min > grid[i].coord.lon) lpj_lon_min = grid[i].coord.lon;
      if (lpj_lon_max < grid[i].coord.lon) lpj_lon_max = grid[i].coord.lon;
      if (lpj_lat_min > grid[i].coord.lat) lpj_lat_min = grid[i].coord.lat;
      if (lpj_lat_max < grid[i].coord.lat) lpj_lat_max = grid[i].coord.lat;
    }
    lpj_nlons = (int)((lpj_lon_max - lpj_lon_min) / (config.resolution.lon)) + 1;
    lpj_nlats = (int)((lpj_lat_max - lpj_lat_min) / (config.resolution.lat)) + 1;
    minlonb = lpj_lon_min - (config.resolution.lon * 0.5);
    maxlonb = lpj_lon_max + (config.resolution.lon * 0.5);
    minlatb = lpj_lat_min - (config.resolution.lat * 0.5);
    maxlatb = lpj_lat_max + (config.resolution.lat * 0.5);
    printf("LPJ grid on pe %d: %d cells in bounding box of %dx%d cells, %g < lon < %g, %g < lat < %g\n",
           pe, // MPI task id
           config.ngridcell,
           lpj_nlons, lpj_nlats, minlonb, maxlonb, minlatb, maxlatb);
    printf("FMS grid on pe %d: %d cells in bounding box of %dx%d cells, %g < lon < %g, %g < lat < %g\n",
           pe, // MPI task id
           (*nlons_in) * (*nlats_in),
           *nlons_in, *nlats_in, *minlonb_in, *maxlonb_in, *minlatb_in, *maxlatb_in);
    //assert(minlonb >= 0.0 && minlonb <= 360.0);
    //assert(maxlonb >= 0.0 && maxlonb <= 360.0);
    //assert(minlatb >= -90.0 && minlatb <= 90.0);
    //assert(maxlatb >= -90.0 && maxlatb <= 90.0);
  }


  // Domain decomposition example for 20 PEs (== MPI tasks):
  //
  // FMS: 5 x 4 = 20 tasks
  //LAND MODEL domain decomposition
  // whalo =    0, ehalo =    0, shalo =    0, nhalo =    0
  //X-AXIS =  144 144 144 144 144
  //Y-AXIS =   90  90  90  90
  //
  // indices in F90 notation 1..N
  //  j\i|  1  144 145  288 289  432 433  576 577  720
  // ----+--------+--------+--------+--------+--------+ 90
  // 360 |        |        |        |        |        |
  //     | pe 15  | pe 16  | pe 17  | pe 18  | pe 19  |
  // 271 |        |        |        |        |        |
  //     +--------+--------+--------+--------+--------+ 45
  // 270 |        |        |        |        |        |
  //     | pe 10  | pe 11  | pe 12  | pe 13  | pe 14  |
  // 181 |        |        |        |        |        |
  //     +--------+--------+--------+--------+--------+  0
  // 180 |        |        |        |        |        |
  //     | pe 5   | pe 6   | pe 7   | pe 8   | pe 9   |
  //  91 |        |        |        |        |        |
  //     +--------+--------+--------+--------+--------+ -45
  //  90 |        |        |        |        |        |
  //     | pe 0   | pe 1   | pe 2   | pe 3   | pe 4   |
  //   1 |        |        |        |        |        |
  //     +--------+--------+--------+--------+--------+ -90
  //     0       72       144      216      288      360 deg

  // LPJ
  // pe  0: 4545 cells in bounding box of 720x7 cells, -180 < lon < 180, -90 < lat < -86.5
  // pe  1: 4544 cells in bounding box of 720x7 cells, -180 < lon < 180, -87 < lat < -83.5
  // pe  2: 4544 cells in bounding box of 720x7 cells, -180 < lon < 180, -84 < lat < -80.5
  // pe  3: 4544 cells in bounding box of 720x8 cells, -180 < lon < 180, -81 < lat < -77
  // pe  4: 4544 cells in bounding box of 618x11 cells, -144 < lon < 165, -77.5 < lat < -72
  // pe  5: 4544 cells in bounding box of 504x79 cells, -75 < lon < 177, -72.5 < lat < -33
  // pe  6: 4544 cells in bounding box of 450x30 cells, -72 < lon < 153, -33.5 < lat < -18.5
  // pe  7: 4544 cells in bounding box of 456x29 cells, -81 < lon < 147, -19 < lat < -4.5
  // pe  8: 4544 cells in bounding box of 456x28 cells, -84 < lon < 144, -5 < lat < 9
  // pe  9: 4544 cells in bounding box of 456x23 cells, -105 < lon < 123, 8.5 < lat < 20
  // pe 10: 4544 cells in bounding box of 468x18 cells, -114 < lon < 120, 19.5 < lat < 28.5
  // pe 11: 4544 cells in bounding box of 504x15 cells, -117 < lon < 135, 28 < lat < 35.5
  // pe 12: 4544 cells in bounding box of 528x15 cells, -123 < lon < 141, 35 < lat < 42.5
  // pe 13: 4544 cells in bounding box of 534x13 cells, -123 < lon < 144, 42 < lat < 48.5
  // pe 14: 4544 cells in bounding box of 576x11 cells, -129 < lon < 159, 48 < lat < 53.5
  // pe 15: 4544 cells in bounding box of 654x12 cells, -165 < lon < 162, 53 < lat < 59
  // pe 16: 4544 cells in bounding box of 690x10 cells, -165 < lon < 180, 58.5 < lat < 63.5
  // pe 17: 4544 cells in bounding box of 720x9 cells, -180 < lon < 180, 63 < lat < 67.5
  // pe 18: 4544 cells in bounding box of 720x11 cells, -180 < lon < 180, 67 < lat < 72.5
  // pe 19: 4544 cells in bounding box of 720x24 cells, -180 < lon < 180, 72 < lat < 84

  // What we really need to use is the FMS boundary box, but with LPJ resolution.
  {
    double nlons_lpj_in_fms = (*maxlonb_in - *minlonb_in) / config.resolution.lon;
    double nlats_lpj_in_fms = (*maxlatb_in - *minlatb_in) / config.resolution.lat;
    printf("LPJ resolution in FMS domain boundaries yields %gx%g cells\n",
           nlons_lpj_in_fms, nlats_lpj_in_fms);
    // make sure we embed integral number of LPJ cells into the FMS domain
    printf("nlons_lpj_in_fms %g (double)(int)(round(nlons_lpj_in_fms))) %g\n", nlons_lpj_in_fms, (double)(int)(round(nlons_lpj_in_fms)));
    printf("nlats_lpj_in_fms %g (double)(int)(round(nlats_lpj_in_fms))) %g\n", nlats_lpj_in_fms, (double)(int)(round(nlats_lpj_in_fms)));
    assert(double_eq(nlons_lpj_in_fms, (double)(int)(round(nlons_lpj_in_fms)), epsilon));
    assert(double_eq(nlats_lpj_in_fms, (double)(int)(round(nlats_lpj_in_fms)), epsilon));
    // we require land_lad and LPJ to run on same resolution
    assert((int)round(nlons_lpj_in_fms) == *nlons_in);
    assert((int)round(nlats_lpj_in_fms) == *nlats_in);
    nlons = (int)round(nlons_lpj_in_fms);
    nlats = (int)round(nlats_lpj_in_fms);
    assert(ie-is+1 == nlons);
    assert(je-js+1 == nlats);
  }
  // cell centers of border cells in FMS domain in LPJ resolution
  lpj_lon_min = *minlonb_in + config.resolution.lon * 0.5;
  lpj_lon_max = *maxlonb_in - config.resolution.lon * 0.5;
  lpj_lat_min = *minlatb_in + config.resolution.lat * 0.5;
  lpj_lat_max = *maxlatb_in - config.resolution.lat * 0.5;

  // now we should have all information to set up the actual mapping (??)
  printf("lpj_lon_min %f lpj_lat_min %f config.nall %d nlons*nlats %d\n",
         lpj_lon_min, lpj_lat_min, config.nall, nlons*nlats);

  // set up CPL mappings
  { int *cpl_xlen, *cpl_ylen;
    int *cpl_ranks;
    Cpl_coord *cpl_coords;
    printf("tasks layout %d x %d\n", layout[0], layout[1]);
    cpl_xlen = newvec(int, layout[0]);
    for (i = 0; i < layout[0]; i++) {
      cpl_xlen[i] = nlons;  // all FMS domains have same number of columns
#ifdef TEST_CPL
      printf("cpl_xlen[%d] %d\n", i, cpl_xlen[i]);
#endif
    }
    cpl_ylen = newvec(int, layout[1]);
    for (j = 0; j < layout[1]; j++) {
      cpl_ylen[j] = nlats; // all FMS domains have same number of rows
#ifdef TEST_CPL
      printf("cpl_ylen[%d] %d\n", j, cpl_ylen[j]);
#endif
    }
#ifdef TEST_CPL
    for (i = 0; i < layout[0]; i++) printf("cpl_xlen[%d] %d\n", i, cpl_xlen[i]);
    for (j = 0; j < layout[1]; j++) printf("cpl_ylen[%d] %d\n", j, cpl_ylen[j]);
#endif
    cpl_ranks = newvec(int, layout[0]*layout[1]);
    for (j = 0; j < layout[1]; j++) {
      for (i = 0; i < layout[0]; i++) {
#ifdef TEST_CPL
        printf("cpl_ranks [%d,%d] = %d\n", i, j, i+layout[0]*j);
#endif
        cpl_ranks[i+layout[0]*j] = i+layout[0]*j;
      }
    }
    cpl_coords = newvec(Cpl_coord, config.ngridcell);
    for (i = 0; i < config.ngridcell; i++) {
      // transform -180 .. 180 to 0 .. 360
      double lon = grid[i].coord.lon < 0 ? grid[i].coord.lon + 360.0 : grid[i].coord.lon;
      int global_i, global_j; // indices of this cell in global land_lad grid
      global_i = (int)((lon - glon_min) / config.resolution.lon + 0.5);
      global_j = (int)((grid[i].coord.lat - glat_min) / config.resolution.lat + 0.5);
      cpl_coords[i].x = global_i;
      cpl_coords[i].y = global_j;
#ifdef TEST_CPL
      { // just for debugging
        int local_i, local_j; // indices of this cell in its local land_lad grid domain
        int lad_pe_i, lad_pe_j; // 2-D indices of the land_lad domain in the global grid.
        int lad_pe; // PE number of the land_lad domain
        lad_pe_i = global_i / nlons; //assert(0 <= lad_pe_i); assert(lad_pe_i < layout[0]);
        lad_pe_j = global_j / nlats; //assert(0 <= lad_pe_j); assert(lad_pe_j < layout[1]);
        lad_pe = lad_pe_i + lad_pe_j * layout[0];
        //assert(0 <= lad_pe);
        //assert(lad_pe < layout[0]*layout[1]);
        local_i = global_i - lad_pe_i * nlons;
        local_j = global_j - lad_pe_j * nlats;
        printf("map pe %2d cell %5d lon %7.2f=%6.2f lat %6.2f -> %3d %3d pe %dx%d= %2d at local i/j %3d %3d\n",
               pe, i, grid[i].coord.lon, lon, grid[i].coord.lat,
               global_i, global_j,
               lad_pe_i, lad_pe_j, lad_pe,
               local_i, local_j);
      } // just for debugging
#endif /* TEST_CPL */
    }
    cpl        = cpl_init(cpl_coords,    config.ngridcell, cpl_xlen, cpl_ylen, cpl_ranks, layout[0], layout[1], comm);
    free(cpl_coords); free(cpl_ranks); free(cpl_ylen); free(cpl_xlen);
  }
  if (!cpl) { fprintf(stderr, "Error: cpl_init() failed\n"); exit(1); }
  printf("cpl_init() got cpl_size %d\n", cpl_size(cpl));
  for (i = 0; i < npes; i++) {
    printf("  for pe %2d: inlen %5d indisp %5d outlen %5d outdisp %5d\n",
           i, cpl->inlen[i], cpl->indisp[i], cpl->outlen[i], cpl->outdisp[i]);
    for (j = cpl->indisp[i]; j < cpl->indisp[i] + cpl->inlen[i]; j++) {
      if (j < 0 || j >= config.ngridcell) printf("index %d outside src array\n", j);
    }
    for (j = cpl->outdisp[i]; j < cpl->outdisp[i] + cpl->outlen[i]; j++) {
      if (j < 0 || j >= cpl_size(cpl)) printf("index %d outside dst array\n", j);
    }
  }

  // number of cells sent/received between tasks in the 5x4 example
  //recv 0    1    2    3    4    5    6    7    8    9   10   11   12   13   14   15   16   17   18   19
  //-----------------------------------------------------------------------------------------------------
  //send
  // 0 1008  945  864  864  864        
  // 1  864  927 1008  881  864
  // 2  864  864  864  991  961
  // 3 1008 1008  804  891  833
  // 4 1440 1433  378  723  570
  // 5 1062 1501  306  102  438   72  193  276   60  534
  // 6                          1089 1625  438      1392
  // 7                          1659  635   78  246 1926 
  // 8                           564  349       180  600 1494  313       252  792
  // 9                                                   2328  968       450  798
  //10                                                   2189 1437       402  516
  //11                                                   1897 1477       918  252
  //12                                                   1440 1574      1308  222
  //13                                                    672  705       612   78  875  828       612  162
  //14                                                                            1549 1477   30 1104  384
  //15                                                                            1272 1484  186 1212  390
  //16                                                                            1158 1395  834 1001  156
  //17                                                                             960 1152 1023  951  458
  //18                                                                             511 1313  901 1164  655
  //19                                                                             362 1033   49 1068 2032

  //mevap_yesterday = newvec(Real, config.ngridcell);
  //mtransp_yesterday = newvec(Real, config.ngridcell);
  //mevap_lake_yesterday = newvec(Real, config.ngridcell);
  //mevap_res_yesterday = newvec(Real, config.ngridcell);
  //minterc_yesterday = newvec(Real, config.ngridcell);

  *global_number_lpj_cells = config.nall;
}


/**
 * Second step of LPJ initialisation. The Fortran side now has allocated
 * some data structures based on the information returned by lpj_init_().
 * Here we fill in those structures.
 */
#ifdef __cplusplus
extern "C"
#endif
void lpj_init_grid_interpolation_
(
 /*! output parameters */
 double *land_mask, /*< 2-D array, 1=land cell, 0 elsewhere */
 double *lpj_cell_lons,
 double *lpj_cell_lats,
 int *lpj_cell_next,
 /*! and finally some interface-checking args again */
const int *const fse
 ) {
  int i;
  double *src = alloca(sizeof(double) * config.ngridcell);
  double *dst = alloca(sizeof(double) * cpl_size(cpl));

  // F90 -> C interface checking
  if (! double_eq(land_mask[(is-is) + (js-js)*(ie-is+1)], 4242.42, epsilon)) {
    printf("Error: 2-D test array corner [%d,%d] %f != 4242.42\n", is-is, js-js, land_mask[(is-is) + (js-js)*(ie-is+1)]);
    fflush(stdout); abort();
  }
  if (! double_eq(land_mask[(ie-is) + (js-js)*(ie-is+1)], 4343.43, epsilon)) {
    printf("Error: 2-D test array corner [%d,%d] %f != 4343.43\n", ie-is, js-js, land_mask[(ie-is) + (js-js)*(ie-is+1)]);
    fflush(stdout); abort();
  }
  if (! double_eq(land_mask[(is-is) + (je-js)*(ie-is+1)], 4444.44, epsilon)) {
    printf("Error: 2-D test array corner [%d,%d] %f != 4444.44\n", is-is, je-js, land_mask[(is-is) + (je-js)*(ie-is+1)]);
    fflush(stdout); abort();
  }
  if (! double_eq(land_mask[(ie-is) + (je-js)*(ie-is+1)], 4545.45, epsilon)) {
    printf("Error: 2-D test array corner [%d,%d] %f != 4545.45\n", ie-is, je-js, land_mask[(ie-is) + (je-js)*(ie-is+1)]);
    fflush(stdout); abort();
  }
  if (*fse != 4711) { printf("Error: %s : parameter test failed: 4711 != %d\n", __FUNCTION__, *fse); fflush(stdout); abort(); }

  // set up land-sea-mask
  for (i = 0; i < nlons*nlats; i++) land_mask[i] = -1.0;

  for (i = 0; i < config.ngridcell; i++) { // loop over local cells
    //double lon = grid[i].coord.lon < 0 ? grid[i].coord.lon + 360.0 : grid[i].coord.lon;
    //int ilon = (int)((lon - glon_min) / config.resolution.lon + 0.5);
    //int jlat = (int)((grid[i].coord.lat - glat_min) / config.resolution.lat + 0.5);
    //printf("on pe %2d cell %5d lon %7.2f=%6.2f lat %6.2f -> %3d %3d\n",
    //       pe, i, grid[i].coord.lon, lon, grid[i].coord.lat,
    //       ilon, jlat);
    //assert(ilon >= 0 && ilon < nlons);
    //assert(jlat >= 0 && jlat < nlats);
    //printf("on pe %2d cell %5d lon %7.2f=%6.2f lat %6.2f -> cpl_src_index %d\n",
    //       pe, i, grid[i].coord.lon, lon, grid[i].coord.lat, cpl_src_index(cpl,i));
    //
    // Insert our PE number (== MPI task number)  so that we can visualize the
    // mapping between LPJ and FMS domains.
    // Offset by 1 to avoid 0, because this land mask is used as land-sea mask
    // in the diag manager lateron, and there, 0 means ocean.
    //TODO: for production , just set all values to 1 here.
    src[cpl_src_index(cpl,i)] = pe + 1;
  }
#ifdef USE_MPI
#ifdef TEST_CPL
  printf("on pe %2d MPI_Alltoallv: \n", pe);
  for (i = 0; i < npes; i++) {
    int j;
    printf("  for pe %2d: inlen %5d indisp %5d to %5d\n", i, cpl->inlen[i], cpl->indisp[i], cpl->indisp[i] + cpl->inlen[i]);
    for (j = cpl->indisp[i]; j < cpl->indisp[i] + cpl->inlen[i]; j++) {
      if (j < 0 || j >= config.ngridcell) printf("index %d outside src array\n", j);
      printf("  src[%d] %f\n", j, src[j]);
    }
  }
#endif /* TEST_CPL */
  cpl_toarr(cpl,src,dst,MPI_DOUBLE);
#ifdef TEST_CPL
  for (i = 0; i < npes; i++) {
    int j;
    printf("  got from pe %2d outlen %5d outdisp %5d to %5d\n", i, cpl->outlen[i], cpl->outdisp[i], cpl->outdisp[i] + cpl->outlen[i]);
    for (j = cpl->outdisp[i]; j < cpl->outdisp[i] + cpl->outlen[i]; j++) {
      if (j < 0 || j >= cpl_size(cpl)) printf("index %d outside dst array\n", j);
      printf("  dst[%d] %f\n", j, dst[j]);
    }
  }
#endif /* TEST_CPL */
#else
  cpl_toarr(cpl,src,dst,sizeof(double));
#endif
  for (i=0; i < cpl_size(cpl); i++) {
    land_mask[cpl_xcoord(cpl,i) + cpl_ycoord(cpl,i) * nlons] = dst[i];
  }
#ifdef TEST_CPL
  {
    int j;
    const char c[] = { '.', '0','1','2','3','4','5','6','7','8','9',
                       'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z' };
    for (j = nlats-1; j >=0; j--) {
      for (i = 0; i < nlons; i++) {
        assert(j*nlons+i < nlons*nlats);
        assert(1+(int)(land_mask[j*nlons+i]) >= 0);
        assert(1+(int)(land_mask[j*nlons+i]) < sizeof(c)/sizeof(c[0]));
        printf("%c",c[1+(int)(land_mask[j*nlons+i])]);
      }
      printf("\n");
    }
  }
#endif /* TEST_CPL */
  fflush(stdout); fflush(stderr);
  MPI_Barrier(comm);

  // assemble runoff map information
  printf("startgrid %d ngridcell %d nall %d\n",
         config.startgrid, config.ngridcell, config.nall);
  for (i = 0; i < config.ngridcell; i++) {
    lpj_cell_lons[config.startgrid + i] = grid[i].coord.lon;
    lpj_cell_lats[config.startgrid + i] = grid[i].coord.lat;
    lpj_cell_next[config.startgrid + i] = grid[i].discharge.next;
    //printf("discharge map %5d %7.2f %7.2f -> %5d\n",
    //       i, grid[i].coord.lon, grid[i].coord.lat, grid[i].discharge.next);
  }
} /* lpj_init_grid_interpolation_() */


/** helper function for conversion from FMS/land_lad domain to LPJ grid cells */
static inline void
cpl_fms_to_lpj(const double *from_fms_in, //< input data on 2-D fms compute domain
               double *to_lpj_out         //< output data, 1-D, for each LPJ grid cell
               ) {
  int i;
  double *buf_lpj;
  double *buf_cpl;

  fmsclock_begin_(&lpj_update_cpl_clock_id);
  buf_lpj = alloca(sizeof(double) * config.ngridcell);
  buf_cpl = alloca(sizeof(double) * cpl_size(cpl));

  for (i=0; i < cpl_size(cpl); i++) {
    assert(0 <= cpl_xcoord(cpl,i) + cpl_ycoord(cpl,i) * nlons);
    assert(cpl_xcoord(cpl,i) + cpl_ycoord(cpl,i) * nlons < nlons*nlats);
    buf_cpl[i] = from_fms_in[cpl_xcoord(cpl,i) + cpl_ycoord(cpl,i) * nlons];
    //buf_cpl[i] = pe; for testing this function and visualisation of the domain mapping
  }
#ifdef USE_MPI
  cpl_togrid(cpl, buf_cpl, buf_lpj, MPI_DOUBLE);
#else
  cpl_togrid(cpl, buf_cpl, buf_lpj, sizeof(double));
#endif
  for (i = 0; i < config.ngridcell; i++) {
    assert(0 <= cpl_src_index(cpl,i));
    assert(cpl_src_index(cpl,i) < config.ngridcell);
    to_lpj_out[i] = buf_lpj[cpl_src_index(cpl,i)];
  }
  fmsclock_end_(&lpj_update_cpl_clock_id);
}

/** helper function for conversion from LPJ grid cells to FMS/land_lad domain */
static inline void
cpl_lpj_to_fms(const double *from_lpj, //< input data, 1-D, for each LPJ grid cell
               double *to_fms          //< output data on 2-D fms compute domain
               ) {
  int i;
  static double *buf_lpj = NULL;
  static double *buf_cpl = NULL;

  fmsclock_begin_(&lpj_update_cpl_clock_id);
  if (!buf_lpj) buf_lpj = malloc(sizeof(double) * config.ngridcell);
  if (!buf_cpl) buf_cpl = malloc(sizeof(double) * cpl_size(cpl));

  for (i = 0; i < config.ngridcell; i++) {
    assert(0 <= cpl_src_index(cpl,i));
    assert(cpl_src_index(cpl,i) < config.ngridcell);
    buf_lpj[cpl_src_index(cpl,i)] = from_lpj[i];
    //printf("cpl_lpj_to_fms: buf_lpj[%d] = from_lpj[%d] = %f\n", cpl_src_index(cpl,i), i, buf_lpj[cpl_src_index(cpl,i)]);
  }
#ifdef USE_MPI
  cpl_toarr(cpl, buf_lpj, buf_cpl, MPI_DOUBLE);
#else
  cpl_toarr(cpl, buf_lpj, buf_cpl, sizeof(double));
#endif
  for (i=0; i < cpl_size(cpl); i++) {
    assert(0 <= cpl_xcoord(cpl,i) + cpl_ycoord(cpl,i) * nlons);
    assert(cpl_xcoord(cpl,i) + cpl_ycoord(cpl,i) * nlons < nlons*nlats);
    to_fms[cpl_xcoord(cpl,i) + cpl_ycoord(cpl,i) * nlons] = buf_cpl[i];
    //printf("cpl_lpj_to_fms: to_fms[%d] = buf_cpl[%d] = %f\n", cpl_xcoord(cpl,i) + cpl_ycoord(cpl,i) * nlons, i);
  }
  fmsclock_end_(&lpj_update_cpl_clock_id);
}


/**
 * Init daily output from values that are saved in the LPJ restart file.
 * We must pass back meaningful values to land_lad/FMS already in the first time step,
 * even before the invocation of lpj_update.
 * This can be done only after all else has been initialised above.
 * Note that the same code for writing out diagnostics and units conversion
 * appears in lpj_update below.
 */
#ifdef __cplusplus
extern "C"
#endif
void lpj_init_output_
  (   /*! pass back results from LPJ */
   double *carbon_flux_out, /* daily carbon flux from LPJ to atmosphere (gC/m2/day) */
   double *q_ca_out,
   double *runoff_out,
   double *roughness_length_out,
   double *surface_temp_out, /* returned in [degC] */
   double *albedo_out,
   double *evap1_out,
   double *dedt_out,
   double *gc_out,
   double *exlpj_out,
   /*! x and y size of the arrays */
   const int *xsize,
   const int *ysize,
   /*! and finally some interface-checking args again */
   const int *const fse
      ) {
  double *tmp_q_ca_out        = alloca(sizeof(double) * config.ngridcell);
  double *tmp_runoff_out      = alloca(sizeof(double) * config.ngridcell);
  //double *tmp_runoff_out = goodnight?alloca(sizeof(double) * config.ngridcell):NULL;
  int cell;
#ifdef TEST_CPL
  double q_ca_min, q_ca_max, q_ca_mean = 0;
#endif

  if (*fse != 4711) { printf("Error: %s : parameter test failed: 4711 != %d\n", __FUNCTION__, *fse); fflush(stdout); abort(); }

  assert(*xsize == nlons);
  assert(*ysize == nlats);

  { // another interface test
    int i, j;
    for (j = 0; j < nlats; j++) {
      for (i = 0; i < nlons; i++) {
        assert(j*nlons+i < nlons*nlats);
        assert(q_ca_out[j*nlons+i] == 0);
      }
    }
  }


#ifdef TEST_CPL
  // Test: fill all cell values with the LPJ domain number
  for (cell=0; cell<config.ngridcell; cell++) {
    tmp_q_ca_out[cell] = pe+1;
  }
  cpl_lpj_to_fms(tmp_q_ca_out, q_ca_out);
  {
    int i, j;
    const char c[] = { '.', '0','1','2','3','4','5','6','7','8','9',
                       'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
                       '*' };
    printf("%s TEST_CPL domain mapping\n", __FUNCTION__);
    for (j = nlats-1; j >=0; j--) {
      for (i = 0; i < nlons; i++) {
        assert(j*nlons+i < nlons*nlats);
        int cnum=(int)(q_ca_out[j*nlons+i]);
        if (cnum < 0) cnum = 0;
        if (cnum >= sizeof(c)/sizeof(c[0])) cnum = sizeof(c)/sizeof(c[0])-1;
        printf("%c",c[cnum]);
      }
      printf("\n");
    }
  }
#endif /* TEST_CPL */

#ifdef TEST_CPL
  q_ca_min = 1e10, q_ca_max = -1e10, q_ca_mean = 0;
  for (cell = 0; cell < *xsize * *ysize; cell++) {
    if (q_ca_out[cell] < q_ca_min) q_ca_min = q_ca_out[cell];
    if (q_ca_out[cell] > q_ca_max) q_ca_max = q_ca_out[cell];
    q_ca_mean += q_ca_out[cell];
  }
  q_ca_mean /= *xsize * *ysize;
  printf("%s q_ca_out inital values       : min %g max %g mean %g\n", __FUNCTION__, q_ca_min, q_ca_max, q_ca_mean);

  q_ca_min = 1e10, q_ca_max = -1e10, q_ca_mean = 0;
  for (cell=0; cell<config.ngridcell; cell++) {
    if (grid[cell].q_ca < q_ca_min) q_ca_min = grid[cell].q_ca;
    if (grid[cell].q_ca > q_ca_max) q_ca_max = grid[cell].q_ca;
    q_ca_mean += grid[cell].q_ca;
  }
  q_ca_mean /= config.ngridcell;
  printf("%s q_ca/dwflux from restart file: min %g max %g mean %g\n", __FUNCTION__, q_ca_min, q_ca_max, q_ca_mean);
#endif /* TEST_CPL */

  // Copy daily output into output variables for land_lad
  for (cell=0; cell<config.ngridcell; cell++) {
    if (grid[cell].skip) {
      tmp_q_ca_out[cell] = 0.0;
    } else {
      tmp_q_ca_out[cell] = grid[cell].q_ca;
    } /* if (!grid[cell].skip) */
    // collect runoff also from skipped cells!
    tmp_runoff_out[cell] = grid[cell].discharge.dfout/86400.0; // [l/day] -> [l/s] = [kg/s]
  } /* loop over cells */

  cpl_lpj_to_fms(tmp_q_ca_out, q_ca_out);
  cpl_lpj_to_fms(tmp_runoff_out, runoff_out);

#ifdef TEST_CPL
  q_ca_min = 1e10, q_ca_max = -1e10, q_ca_mean = 0;
  for (cell = 0; cell < *xsize * *ysize; cell++) {
    if (q_ca_out[cell] < q_ca_min) q_ca_min = q_ca_out[cell];
    if (q_ca_out[cell] > q_ca_max) q_ca_max = q_ca_out[cell];
    q_ca_mean += q_ca_out[cell];
  }
  q_ca_mean /= *xsize * *ysize;
  printf("%s q_ca_out after cpl_lpj_to_fms: min %g max %g mean %g\n", __FUNCTION__, q_ca_min, q_ca_max, q_ca_mean);
#endif

  /**TODO roughness_length, surface_temp, albedo, evap1, dedt, gc, exlpj */

} /* lpj_init_output_() */



/**
 * Update LPJ by one day's time step.
 * The original main program uses one iterate() function, which loops
 * over the entire simulation time, and for every year a function iterate_year()
 * is invoked. That in turn loops over months, and inside that over days.
 * We must break up the original iterate() function hierarchy so that it does one
 * tiny time step with every update invocation. The update function must then check
 * for new year / new month events to perform the necessary init / finish operations
 * according to the event.
 */
#ifdef __cplusplus
extern "C"
#endif
void lpj_update_
(  const int *const Time_year, const int *const Time_month, const int *const Time_day,
   const int *const Time_hour, const int *const Time_minute, const int *const Time_seconds,
/**     C implementation of LPJmL                                                  \n**/
   /*! get climate status information. Fortran 90 arrays of dimension 0:je-js, 0:ie-is */
   //const double *fprec_in, //< TODO: use liquid and frozen precipitation as given by FMS
   //const double *lprec_in,
   const double *prec_in, //< currently, LPJ expects total precipitation and calculates liquid/frozen itself
   const double *temp_mean_in,
   const double *swdown_in,
   const double *lwnet_in,
   const double *t_flux_in,
   const double *qflux_in,
   const double *dedq_in,
   const double *dhdt_in,
   const double *drdt_in,
   const double *drag_q_in,
   const double *p_surf_in,
   const double *wind_in,
   const double *co2_in,
   /*! pass back results from LPJ */
   double *carbon_flux_out, /* daily carbon flux from LPJ to atmosphere (gC/m2/day) */
   double *q_ca_out,
   double *runoff_out,
   double *roughness_length_out,
   double *surface_temp_out,
   /*! TODO: folia protective cover [%] */
   /*! TODO: LAI */
   double *albedo_out,
   double *evap1_out,
   double *dedt_out,
   double *gc_out,
   double *exlpj_out,
   /*! minimal interface check */
   int *dt_fast_int,
   const int *const fortytwo /*< last arg, minimal safety against interface changes */
  ) {
  /* Starting simulation */

  /* broken up invocation of loop over the months, see above */

  /* main loop over spinup + simulation years  */
  /* for(year=config.firstyear-config.nspinup;year<=config.lastyear;year++) */
  {
    /*const int*/ year = *Time_year;
    const int month = *Time_month - 1; /* dailyclimate.c: month (0..11) */
    const int dayofmonth = *Time_day - 1;
   int year_rnd;
    int s;
    Stand *stand;
    static int yesterday = -1;
    // TODO: make sure criteria for goodnight etc match those used in lpj_driver.F90 _exactly_
    // or: get those flags as input parameters from caller
    const Bool goodmorning = yesterday != dayofmonth;
    const Bool goodnight = (*Time_hour*3600+*Time_minute*60+*Time_seconds + *dt_fast_int) >= NSECONDSDAY; /* last timestep of the day */
    const Bool newmoon = goodmorning && 0 == dayofmonth;
    const Bool monthend = goodnight && dayofmonth == ndaymonth[month] - 1;
    const Bool happynewyear = newmoon && 0 == month;
    const Bool silvester = (goodnight && yesterday != -1  /* not before the first simulation year */
                             && 11 == month && 30 == dayofmonth);
    Real dt_fast;
    assert(42 == *fortytwo); /* make sure that number of parameters matches between F90 and C */

    /* TODO: the ``real work'' of daily updates is called only once per day
     * (at goodnight time)
     * Thus we should possibly check right here if goodnight is true,
     * and just return otherwise.
     */
    dt_fast=*dt_fast_int;
    printf("DT_FAST*: %g, dt_fast_old %g \n", dt_fast, dt_fast);
    if (isroot(config))
      printf("%s %04d-%02d-%02d-%02d:%02d:%02d %s %s %s %s %s %s\n",
             __FUNCTION__,
             year, month, dayofmonth, *Time_hour, *Time_minute, *Time_seconds,
             goodmorning ? "goodmorning" : "",
             goodnight   ? "goodnight"   : "",
             newmoon     ? "newmoon"     : "",
             monthend    ? "monthend"    : "",
             happynewyear? "happynewyear": "",
             silvester   ? "silvester"   : "");

    /* some things must be done at start of a new year only */
      if (!config.co2_from_file || input.climate->co2.data!=NULL) // invoke getco2() when reading from real file, not FMS
      co2=getco2(input.climate,year,&config); /* get atmospheric CO2 concentration */
    if (happynewyear) { /* from iterate() */
      printf("LPJmL_CO2: %g\n",co2);
#ifdef IMAGE
      if(year>=config.start_imagecoupling)
        co2=receive_image_co2(&config);
      else
#endif
      /* else: use CO2 as given by LPJ coupler - done below in daily loop-over-cells */

      //if(year<input.climate->firstyear) /* are we in spinup phase? */
      ///* yes, let climate data point to stored data */
//#ifdef STORECLIMATE
      //  moveclimate(input.climate,store,
      //              (year-config.firstyear+config.nspinup) % config.nspinyear);
//#else
      //getclimate(input.climate,grid,input.climate->firstyear+(year-config.firstyear+config.nspinup) % config.nspinyear,&config);
//#endif
      //else
      {
#ifdef STORECLIMATE
        if(year==input.climate->firstyear && config.nspinup)
        {
          /* restore climate data pointers to initial data */
          input.climate->data=data_save;
          freeclimatedata(&store); /* free data not used anymore */
        }
#endif
        /* read climate from files */
#ifdef IMAGE
        if(year>=config.start_imagecoupling)
        {
          if(receive_image_climate(input.climate,grid,year,&config))
          {
            fprintf(stderr,"ERROR104: Simulation stopped in receive_image_climate().\n");
            fflush(stderr);
            /*break;*/ /* leave time loop */ abort();
          }
        }
        else
#endif
        year_rnd=config.firstyear+(int)(30*drand48());
#ifdef USE_MPI
        MPI_Bcast(&year_rnd,1,MPI_INT,0,config.comm);
#endif
          if(getclimate(input.climate,grid,year_rnd,&config))
          {
            fprintf(stderr,"ERROR104: Simulation stopped in getclimate().\n");
            fflush(stderr);
            /*break;*/ /* leave time loop */ abort();
          }
      }
      if(input.landuse!=NULL)
      {
        calc_seasonality(grid,npft,ncft,&config);
        if(config.withlanduse==CONST_LANDUSE || config.withlanduse==ALL_CROPS) /* constant landuse? */
          landuse_year=config.landuse_year_const;
        else if(year>2000)			/* 2 lines added to use year 2000 landuse after 2000 if climate data is used */
          landuse_year=2000;
        else
          landuse_year=year;
        /* under constand landuse also keep wateruse at landuse_year_const */
        if(config.withlanduse==CONST_LANDUSE)
          wateruse_year=config.landuse_year_const;
        else wateruse_year=year;
#ifdef IMAGE
        if(year>=config.start_imagecoupling)
        {
          if (receive_image_data(grid,npft,ncft,&config))
          {
            fprintf(stderr,"ERROR104: Simulation stopped in receive_image_data().\n");
            fflush(stderr);
            /*break;*/ /* leave time loop */ abort();
          }
        }
        else
#endif
          /* read landuse pattern from file */
          if(getlanduse(input.landuse,grid,landuse_year,ncft,&config))
          {
            fprintf(stderr,"ERROR104: Simulation stopped in getlanduse().\n");
            fflush(stderr);
            /*break;*/ /* leave time loop */ abort();
          }
        if(config.reservoir)
          allocate_reservoir(grid,year,&config);
      }
      if(input.wateruse!=NULL && input.landuse!=NULL)
      {
        /* read wateruse data from file */
        if(getwateruse(input.wateruse,grid,wateruse_year,&config))
        {
          fprintf(stderr,"ERROR104: Simulation stopped in getwateruse().\n");
          fflush(stderr);
          /*break;*/ /* leave time loop */ abort();
        }
      }
      if(config.ispopulation)
      {
        if(readpopdens(input.popdens,year,grid,&config))
        {
          fprintf(stderr,"ERROR104: Simulation stopped in getpopdens().\n");
          fflush(stderr);
          /*return year*/ abort();
        }
      }
      if (config.prescribe_landcover != NO_LANDCOVER)
      {
        if (readlandcover(input.landcover,grid,year,&config))
        {
          fprintf(stderr,"ERROR104: Simulation stopped in readlandcover().\n");
          fflush(stderr);
          abort();
        }
      }

      if(year>=config.firstyear)
        openoutput_yearly(output,year,&config);

    } /* if (happynewyear) */

    /* perform iteration for one year */
    /* broken up iterateyear_river() to be called on every time step */
      /*iterateyear_river(output,grid,input,co2,npft,ncft,&config,year);*/
      { /* begin iterateyear_river() */
        static Dailyclimate fast;
        static Bool intercrop,istimber;
        /* dailyclimate.c: month (0..11) day (1..365) */
        static int /*month,*/ /*dayofmonth,*/ dayofyear;
        int cell, i;
        static Real popdens=0; /* population density (capita/km2) */

        if (happynewyear) { /* things from iterateyear_river() to be done at begin of year */
          popdens=0;
         
#ifdef IMAGE
          istimber=(config.start_imagecoupling!=INT_MAX);
#else
          istimber=FALSE;
#endif
          intercrop=getintercrop(input.landuse);
          for(cell=0;cell<config.ngridcell;cell++)
          {
            grid[cell].output.adischarge=0;
            grid[cell].output.surface_storage=0;
            grid[cell].daily.tmax=-1e38;
            grid[cell].daily.tmin=1e38;
            grid[cell].daily.prec=grid[cell].daily.swdown=grid[cell].daily.lwnet=grid[cell].daily.windspeed=grid[cell].daily.temp=grid[cell].VPD=0;
            foreachstand(stand,s,grid[cell].standlist)
              stand->dprec=0;
            if(!grid[cell].skip)
            {
              init_annual(grid+cell,npft,ncft,&config);
              if(input.landuse!=NULL)
              {
                if(grid[cell].lakefrac<1)
                {
                  /* calculate landuse change */
                  if(config.laimax_interpolate==CONST_LAI_MAX)
                    laimax_manage(&grid[cell].ml.manage,config.pftpar+npft,npft,ncft,year);
                  if(!config.natural_only && (year>config.firstyear-config.nspinup || config.from_restart))
                    landusechange(grid+cell,config.pftpar,npft,ncft,config.ntypes,
                                  intercrop,istimber,year,config.pft_output_scaled);
                  else if(grid[cell].ml.dam)
                    landusechange_for_reservoir(grid+cell,config.pftpar,npft,istimber,
                                                intercrop,ncft,year);
                }
#ifdef IMAGE
                setoutput_image(grid+cell,ncft);
#endif
              }
              initgdd(grid[cell].gdd,npft);
            } /*gridcell skipped*/
          } /* of for(cell=...) */
          dayofyear = 1;
        } /* if (happynewyear) */

        //day=1;
        //foreachmonth(month)
        if (newmoon) {
          for(cell=0;cell<config.ngridcell;cell++)
          {
            grid[cell].discharge.mfin=grid[cell].discharge.mfout=grid[cell].output.mdischarge=
              grid[cell].ml.mdemand=0.0;
            if(!grid[cell].skip)
            {
              initoutput_monthly(&((grid+cell)->output));
              //// reset yesterday's saved values
              //mevap_yesterday[cell] = mtransp_yesterday[cell] = 0.0;
              //mevap_lake_yesterday[cell] = mevap_res_yesterday[cell] = 0.0;
              //minterc_yesterday[cell] = 0.0; 
              /* Initialize random seed */
              if(israndomprec(input.climate))
                srand48(config.seed+(config.startgrid+cell)*year*month);
              initclimate_monthly(input.climate,&grid[cell].climbuf,cell,month);
#ifdef IMAGE
              monthlyoutput_image(&grid[cell].output,input.climate,cell,month);
#endif

#ifdef DEBUG
              printf("temp = %.2f prec = %.2f wet = %.2f",
                     (getcelltemp(input.climate,cell))[month],
                     (getcellprec(input.climate,cell))[month],
                     (israndomprec(input.climate)) ? (getcellwet(input.climate,cell))[month] : 0);
              if(config.with_radiation)
                printf("lwnet = %.2f swdown = %.2f\n",
                       (getcelllwnet(input.climate,cell))[month],
                       (getcellswdown(input.climate,cell))[month]);
              else
                printf("sun = %.2f\n",(getcellsun(input.climate,cell))[month]);
#endif
            }
          } /* of 'for(cell=...)' */
        } /* if(newmoon) */


       if (goodmorning){
         for(cell=0;cell<config.ngridcell;cell++)
         {
           if(!grid[cell].skip)
           {
             initoutput_daily(&(grid[cell].output.daily));
             if(config.ispopulation)
               popdens=getpopdens(input.popdens,cell);
           }
         }
       }
        /* fast time-step begin */
   // SIGH . daily contains one scalar for each variable. But we need arrays for cpl operations.
          double *tmp_temp_mean = alloca(sizeof(double) * config.ngridcell);
          double *tmp_prec      = alloca(sizeof(double) * config.ngridcell);
          double *tmp_wind      = alloca(sizeof(double) * config.ngridcell);
          double *tmp_swdown    = alloca(sizeof(double) * config.ngridcell);
          double *tmp_lwnet     = alloca(sizeof(double) * config.ngridcell);
          double *tmp_t_flux    = alloca(sizeof(double) * config.ngridcell);
          double *tmp_qflux     = alloca(sizeof(double) * config.ngridcell);
          double *tmp_dedq      = alloca(sizeof(double) * config.ngridcell);
          double *tmp_dhdt      = alloca(sizeof(double) * config.ngridcell);
          double *tmp_drdt      = alloca(sizeof(double) * config.ngridcell);
          double *tmp_drag_q    = alloca(sizeof(double) * config.ngridcell);
          double *tmp_p_surf    = alloca(sizeof(double) * config.ngridcell);
          double *tmp_co2       = NULL;

          double *tmp_carbon_flux_out = alloca(sizeof(double) * config.ngridcell);
          double *tmp_q_ca_out        = alloca(sizeof(double) * config.ngridcell);
          double *tmp_roughness_length_out = alloca(sizeof(double) * config.ngridcell);
          double *tmp_surface_temp_out = alloca(sizeof(double) * config.ngridcell);
          double *tmp_albedo_out = alloca(sizeof(double) * config.ngridcell);
          double *tmp_evap1_out = alloca(sizeof(double) * config.ngridcell);
          double *tmp_dedt_out = alloca(sizeof(double) * config.ngridcell);
          double *tmp_gc_out = alloca(sizeof(double) * config.ngridcell);
          double *tmp_exlpj_out = alloca(sizeof(double) * config.ngridcell);
          double *tmp_diff = alloca(sizeof(double) * config.ngridcell);
          double *tmp_runoff_out = goodnight?alloca(sizeof(double) * config.ngridcell):NULL;

          if (config.co2_from_file && NULL == input.climate->co2.data) // if reading from FMS, not from real file via getco2() above
            tmp_co2             = alloca(sizeof(double) * config.ngridcell);
          cpl_fms_to_lpj(temp_mean_in, tmp_temp_mean);
          cpl_fms_to_lpj(prec_in, tmp_prec);

          cpl_fms_to_lpj(wind_in, tmp_wind);

          //cpl_fms_to_lpj(lightning_in, tmp_lightning);
          cpl_fms_to_lpj(swdown_in, tmp_swdown);

           { // for debugging
            double swdown_mean = 0, swdown_min = 1e6, swdown_max = -1e6;
            int i;
            for (i = 0; i < nlons*nlats; i++) {
              swdown_mean += swdown_in[i];
              if (swdown_min > swdown_in[i]) swdown_min = swdown_in[i];
              if (swdown_max < swdown_in[i]) swdown_max = swdown_in[i];
            }
            printf("swdown_in mean %g min %g max %g\n", swdown_mean/(nlons*nlats), swdown_min, swdown_max);
            //printf("swdown in= %g, swdowntmp= %g\n", max(swdown_in), max(tmp_swdown));
            swdown_mean = 0, swdown_min = 1e6, swdown_max = -1e6;
            for (i = 0; i < config.ngridcell; i++) {
              swdown_mean += tmp_swdown[i];
              if (swdown_min > tmp_swdown[i]) swdown_min = tmp_swdown[i];
              if (swdown_max < tmp_swdown[i]) swdown_max = tmp_swdown[i];
            }
            printf("tmp_swdown mean %g min %g max %g\n", swdown_mean/(nlons*nlats), swdown_min, swdown_max);
          }
          cpl_fms_to_lpj(lwnet_in, tmp_lwnet);
 
         cpl_fms_to_lpj(t_flux_in, tmp_t_flux);

          cpl_fms_to_lpj(qflux_in, tmp_qflux);
 
         { // for debugging
            double qflux_mean = 0, qflux_min = 1e6, qflux_max = -1e6;
            int i;
            for (i = 0; i < nlons*nlats; i++) {
              qflux_mean += qflux_in[i];
              if (qflux_min > qflux_in[i]) qflux_min = qflux_in[i];
              if (qflux_max < qflux_in[i]) qflux_max = qflux_in[i];
            }
            printf("qflux_in mean %g min %g max %g\n", qflux_mean/(nlons*nlats), qflux_min, qflux_max);
            //printf("qflux in= %g, qfluxtmp= %g\n", max(qflux_in), max(tmp_qflux));
            qflux_mean = 0, qflux_min = 1e6, qflux_max = -1e6;
            for (i = 0; i < config.ngridcell; i++) {
              qflux_mean += tmp_qflux[i];
              if (qflux_min > tmp_qflux[i]) qflux_min = tmp_qflux[i];
              if (qflux_max < tmp_qflux[i]) qflux_max = tmp_qflux[i];
            }
            printf("tmp_qflux mean %g min %g max %g\n", qflux_mean/(nlons*nlats), qflux_min, qflux_max);
          }
          cpl_fms_to_lpj(dedq_in, tmp_dedq);

          cpl_fms_to_lpj(dhdt_in, tmp_dhdt);

          cpl_fms_to_lpj(drdt_in, tmp_drdt);

          cpl_fms_to_lpj(drag_q_in, tmp_drag_q);

          cpl_fms_to_lpj(p_surf_in, tmp_p_surf);

          // Get current CO2 value from LPJ_COUPLER / land_lad
          // When receiving data from FMS, we get daily per-cell CO2 values.
          // When reading from file via getco2() we get globally and annually averaged values.
          if (config.co2_from_file && NULL == input.climate->co2.data) // if reading from FMS, not from real file via getco2() above
            cpl_fms_to_lpj(co2_in, tmp_co2); /* get atmospheric CO2 concentration */
          printf("before update_fast");
           for(cell=0;cell<config.ngridcell;cell++)
          {
            if(!grid[cell].skip)
            {
//              if(config.ispopulation)
  //              popdens=getpopdens(input.popdens,cell);
    
     /*    if (isnan(tmp_temp_mean[cell]))
            tmp_temp_mean[cell]=0;
            if (isnan(tmp_prec[cell]))
            tmp_prec[cell]=0;

                if (isnan(tmp_wind[cell]))
            tmp_wind[cell]=0;

     if (isnan(tmp_swdown[cell]))
            tmp_swdown[cell]=0;
      if (isnan(tmp_lwnet[cell]))
            tmp_lwnet[cell]=0;

        if (isnan(tmp_t_flux[cell]))
            tmp_t_flux[cell]=0;

        if (isnan(tmp_qflux[cell]))
            tmp_qflux[cell]=0;

        if (isnan(tmp_dedq[cell]))
            tmp_dedq[cell]=0;

        if (isnan(tmp_dhdt[cell]))
            tmp_dhdt[cell]=0;

        if (isnan(tmp_drdt[cell]))
            tmp_drdt[cell]=0;

        if (isnan(tmp_drag_q[cell]))
            tmp_drag_q[cell]=0;
 if (isnan(tmp_p_surf[cell]))
            tmp_p_surf[cell]=0;
 */
         //    grid[cell].output.dcflux=grid[cell].output.dwflux=0;
        //     initoutput_daily(&(grid[cell].output.daily));
              /* get daily values for temperature, precipitation and sunshine */

              /******* get INPUT from land_lad ****************************************         \n**/
              /* Fill in the daily structure with the current values we got from
               * land_lad. Those values, for which the LPJ input configuration
               * specifies FMS_COUPLER instead of a file name, will be left untouched
               * in dailyclimate(). Values for which a file name is configured, are
               * overwritten.
               */
              fast.temp = tmp_temp_mean[cell];
           
              grid[cell].daily.temp+=fast.temp;
              grid[cell].fasttemp=tmp_temp_mean[cell];
           //   if (grid[cell].coord.lat>=-48.75 && grid[cell].coord.lat<=18.25 && grid[cell].coord.lon>=-105.25 && grid[cell].coord.lon<=-30.25)
           //     fast.prec = tmp_prec[cell]*1.6;
            //  else
                fast.prec = tmp_prec[cell];
              grid[cell].fastprec = tmp_prec[cell];
              grid[cell].daily.prec += fast.prec;
              fast.windspeed = tmp_wind[cell];
              grid[cell].daily.windspeed += fast.windspeed;
	      //daily.lightning = ;
              fast.swdown = tmp_swdown[cell];
	      grid[cell].daily.swdown += fast.swdown;
              fast.lwnet = tmp_lwnet[cell];
              grid[cell].daily.lwnet += fast.lwnet;
              fast.t_flux = tmp_t_flux[cell];
              fast.qflux=tmp_qflux[cell];
              grid[cell].fastqflux=tmp_qflux[cell];
              fast.dedq=tmp_dedq[cell];
              grid[cell].fastdedq=tmp_dedq[cell];
              if(grid[cell].daily.tmax<fast.temp)
                grid[cell].daily.tmax=fast.temp;
              if(grid[cell].daily.tmin>fast.temp)
                grid[cell].daily.tmin=fast.temp;
              fast.dhdt=tmp_dhdt[cell];
              fast.drdt=tmp_drdt[cell];
              fast.drag_q = tmp_drag_q[cell]; //!TODO: read in input configuration from FMS or from file for spinup
              fast.p_surf = tmp_p_surf[cell];

              if (config.co2_from_file && NULL == input.climate->co2.data) { // if reading from FMS, not from real file via getco2() above
                co2 = tmp_co2[cell];
                // co2 = 300; // for testing, set CO2 concentration to arbitrary constant
              }
              /* get daily values for temperature, precipitation and sunshine */

 //             tmp_diff[cell]=grid[cell].output.daily.temp-tmp_temp_mean[cell];


             update_fast(grid+cell,fast,dt_fast,month,&config);
   }
}
printf("after update_fast\n");
//if (grid[cell].coord.lat==-7.25 && grid[cell].coord.lon==-55.75)
 //       printf("daily temp= %g, gldas=%g, ref=%g, deltatemp=%g\n",daily.temp,input.climate->data[0].temp[cell*NDAYYEAR+dayofyear-1], input.climate->data[1].temp[cell*NDAYYEAR+dayofyear-1],tmp_diff[cell]);

              //grid[cell].output.daily.sun=daily.sun; not used for FMS coupling

              /******* now do the MAIN work ****************************************         \n**/

#ifdef DEBUG
              //printf("day=%d cell=%d CO2=%g\n",dayofyear,cell, co2);
#endif
		for (cell=0; cell<config.ngridcell; cell++) {
                 if (cell>1000000){
                   printf("error cell");
                   return;
                 }
            if (!grid[cell].skip) {
              Real temp = 0.0, fracsum = 0.0;

            
                if (grid[cell].q_ca<0.00001){
                  printf("ERR_CLIMBER: qca smaller than 0.00001");
                  tmp_q_ca_out[cell]=0.00001;
                }
                else if (grid[cell].q_ca>0.1){
                  printf("ERR_CLIMBER: qca too large");
                  tmp_q_ca_out[cell]=0.1;
                }
                else
		tmp_q_ca_out[cell] = grid[cell].q_ca;
                if (isnan(tmp_q_ca_out[cell])){
                  printf("ERR_CLIMBER: QCA NAN");
                  tmp_q_ca_out[cell]=0;}
                if (roughnesslength(grid[cell].standlist)<0){
                  printf("ERR_CLIMBER: roughness smaller than 0");
                  tmp_roughness_length_out[cell]=0.0001;
                }
                else if (roughnesslength(grid[cell].standlist)>100){
                  printf("ERR_CLIMBER: roughness larger than 100");
                  tmp_roughness_length_out[cell]=100;  
                }
                else
		tmp_roughness_length_out[cell] = roughnesslength(grid[cell].standlist);
                if (isnan(tmp_roughness_length_out[cell])){
                  printf("ERR_CLIMBER: rough nan");
                  tmp_roughness_length_out[cell]=0;}
        /*        if (grid[cell].standlist->n > 10) {
		  printf("%s %d Error cell %d has too many stands %d\n", __FUNCTION__, __LINE__, cell, grid[cell].standlist->n);
                  fflush(stdout);
                  //exit(42);
                }*/
                foreachstand(stand,s,grid[cell].standlist)
                {
                  // if(grid[cell].standlist->n>10)
                   //  printf("%d %s\n",s,stand->type->name);
      //          if (stand->soil.temp[0]<-120){
       //            printf("ERR_CLIMBER: soiltemp0 too small %g", stand->soil.temp[0]);
        //           stand->soil.temp[0]=-120;
         //       }
          //      if (stand->soil.temp[0]>120){
           //        printf("ERR_CLIMBER: soiltemp0 too large %g", stand->soil.temp[0]);
            //       stand->soil.temp[0]=120;
             //   }

     //           if (stand->soil.temp2<-120){
     //              printf("ERR_CLIMBER: soiltemp2 too small %g",stand->soil.temp2);
      //             stand->soil.temp2=-120;
       //         }
        //        if (stand->soil.temp2>120){
        //           printf("ERR_CLIMBER: soiltemp0 too large %g", stand->soil.temp2);
         //          stand->soil.temp2=120;
               // }

               
                temp += stand->soil.temp2 * stand->frac;
                fracsum += stand->frac;
              }
              if (grid[cell].laketemp<-150){
                printf("ERR_CLIMBER: laketemp too small %g", grid[cell].laketemp);
                grid[cell].laketemp=-150;
              }
                 if (grid[cell].laketemp>150){
                printf("ERR_CLIMBER: laketemp too large %g", grid[cell].laketemp);
                grid[cell].laketemp=150;
              }

        	temp += grid[cell].laketemp * grid[cell].lakefrac;
                fracsum += grid[cell].lakefrac;
              // since we now also consider laketemp and lakefrac, fracsum should be just 1
              tmp_surface_temp_out[cell] = temp /fracsum;
              if (fracsum<0.01)
                tmp_surface_temp_out[cell]=0;
                  if (tmp_surface_temp_out[cell]>120)
                     printf("TEST:  roughness: %g, qca: %g, temp: %g, albedo: %g\n",tmp_roughness_length_out[cell],tmp_q_ca_out[cell],tmp_surface_temp_out[cell],grid[cell].albedo);
               if (tmp_surface_temp_out[cell]<-150){
                 printf("ERR_CLIMBER: final soiltemp too small %g", tmp_surface_temp_out[cell]);
                tmp_surface_temp_out[cell]=-150;
                }
               if (tmp_surface_temp_out[cell]>80){
                 printf("ERR_CLIMBER: final soiltemp too large %g", tmp_surface_temp_out[cell]);
                tmp_surface_temp_out[cell]=80;
                }
               if (isnan(tmp_surface_temp_out[cell]))
               {
                   printf("ERR_CLIMBER: TEMP NAN");
                   printf("roughness: %g, qca: %g, temp: %g, albedo: %g\n",tmp_roughness_length_out[cell],tmp_q_ca_out[cell],tmp_surface_temp_out[cell],grid[cell].albedo);
                   tmp_surface_temp_out[cell]=0;
               }
               
              //tmp_albedo_out[cell] = albedo(&grid[cell]);
              // Now use Erik's new albedo() parameterisation.
              // It requires temperature and precipitation, which we also used for update_daily() above.
              if (grid[cell].albedo<0.05){
  		printf("ERR_CLIMBER: albedo smaller than 0.05");
                tmp_albedo_out[cell]=0.05;
              }
              else if (grid[cell].albedo>0.9){
                printf("ERR_CLIMBER: albedo larger than 0.9");
                tmp_albedo_out[cell]=0.9;
              }
              else        
                tmp_albedo_out[cell] = grid[cell].albedo;
              if (isnan(tmp_albedo_out[cell])){
                  printf("ERR_CLIMBER: ALBEDO NAN");
                  tmp_albedo_out[cell]=0.05;}
            
             // tmp_albedo_out[cell] = grid[cell].albedo;
              tmp_evap1_out[cell] = grid[cell].evap1;
              tmp_dedt_out[cell] = grid[cell].dedt;
              tmp_gc_out[cell] = grid[cell].gc;
              tmp_exlpj_out[cell] = grid[cell].exlpj;
              //tmp_albedo_out[cell] = 0.3;
              //printf("EVAP1: %g \n", tmp_evap1_out[cell]);


            } /* if (!grid[cell].skip) */
          } /* loop over cells */
		/* now we can map the collected LPJ output to the FMS compute domain */
          printf("before cpl_lpj_to_fms task %d\n", pe);
          //cpl_lpj_to_fms(tmp_carbon_flux_out, carbon_flux_out);
          cpl_lpj_to_fms(tmp_q_ca_out, q_ca_out);
          cpl_lpj_to_fms(tmp_roughness_length_out, roughness_length_out);
          cpl_lpj_to_fms(tmp_surface_temp_out, surface_temp_out);
          cpl_lpj_to_fms(tmp_albedo_out, albedo_out);
          cpl_lpj_to_fms(tmp_evap1_out, evap1_out);
          cpl_lpj_to_fms(tmp_dedt_out, dedt_out);
          cpl_lpj_to_fms(tmp_gc_out, gc_out);
          cpl_lpj_to_fms(tmp_exlpj_out, exlpj_out);

           
          printf("end fast time step climber task %d\n", pe);
       /* fast time-step end */ 


        /*foreachdayofmonth(dayofmonth,month)*/  /* for(day=0;day<ndaymonth[month];day++) */
        if (goodnight) {  /* do it only once per day */
              for(cell=0;cell<config.ngridcell;cell++)
          {
            if(!grid[cell].skip)
            {

              grid[cell].output.daily.prec=grid[cell].daily.prec;

               grid[cell].aprec_poem+=grid[cell].daily.prec;
               grid[cell].daily.temp *= dt_fast/NSECONDSDAY;
               grid[cell].daily.swdown *= dt_fast/NSECONDSDAY;
	       grid[cell].daily.lwnet *= dt_fast/NSECONDSDAY;
               grid[cell].daily.windspeed *= dt_fast/NSECONDSDAY;
               grid[cell].output.daily.rh *= dt_fast/NSECONDSDAY;
               grid[cell].VPD *= dt_fast/NSECONDSDAY;
              dailyclimate(&grid[cell].daily,input.climate,&grid[cell].climbuf,grid+cell,cell,dayofyear,
                           month,dayofmonth,config.with_anomaly);
              update_daily(grid+cell,co2,popdens,grid[cell].daily,dayofyear,npft,ncft,
                           year,month,output->withdaily,intercrop,&config);
              //printf("LPJmL_CO2: %g\n",co2);
              grid[cell].output.daily.temp=grid[cell].daily.temp;
            }
          }


          if(config.river_routing)
          {
            if(input.landuse!=NULL || input.wateruse!=NULL)
              withdrawal_demand(grid,&config);

            drain(grid,month,&config);

            if(input.landuse!=NULL || input.wateruse!=NULL)
              wateruse(grid,npft,ncft,&config);

          }

          if(output->withdaily && year>=config.firstyear)
            fwriteoutput_daily(output,grid,dayofyear-1,year,&config);

          /******* prepare OUTPUT for land_lad ****************************************         \n**/

          // Copy daily output into output variables for land_lad
          for (cell=0; cell<config.ngridcell; cell++) {
            if (!grid[cell].skip) {
              //Real temp = 0.0, fracsum = 0.0;

              // in daily_natural.c:
              // output->mnpp+=npp*stand->frac;
              // i.e. it is summed up each day. Thus we need to remember yesterday's
              // value, and compute the difference each day.
              // npp = assimilation of CO2 from atmosphere into soil
              // [g C / m^2 ] - difference per day yields [g C / m^2 / day]
              //
              // in update_daily.c:
              // cell->output.mrh+=hetres*stand->frac;
              // mrh = soil respiration, i.e. source of CO2 going from soil into atmosphere
              // [g C / m^2 ] - difference per day yields [g C / m^2 / day]
              // ---
              // nep = npp - rh
              // in update_monthly.c:
              //grid[cell].balance.nep Pro Zelle  [g C/(m^2 * day)]
              //  positiv: Senke
              //  negativ: Emission
              // grid[cell].coord.area [m^2]
              // But this is done only monthly!
              //
              // grid[cell].output.firec [g C/(m^2 * day)]
              // positiv: Emission
              // negativ: geht nicht
              // Standard: computed annually
              // With SPITFIRE: computed daily
              //
              // grid[cell].output.flux_harvest [g C/(m^2 * day)]
              //   Ernte
              //   geht nicht direkt in die Atmopshaere -> hier unberuecksichtigt erstmal
              //  (in land_lad2 kommt sowas offenbar auch vor)
              //
              // in update_daily.c:
              // grid[cell].output.flux_estab
              //  positiv: Senke
              //  negativ: geht nicht
              // Carbon intake of saplings - additional to npp
              // [ g C / m^2 / day ]
              //
              // Umrechnung g C -> CO2 ueber Mol-Gewicht (12)
              //
              // CO2 fertilization abschalten, CO2 auf festem Wert lassen z.B. 288 ppm
              // ---
              // All the above is now summed up daily in output.dcflux (gC/m2/day)
              // Conversion C -> CO2 is done in the F90 side of this interface.
              //tmp_carbon_flux_out[cell] = grid[cell].output.dcflux;
              //tmp_carbon_flux_out[cell] = pe; for testing the domain mapping

              //// Evaporation: accumulated monthly in cell[].output.mevap.
              ////              The daily value is stored in a local variable in update_daily
              //// WvB: evap contains only the part that comes through sublimation of snow into the atmosphere,
              ////      and is calculated in the permafrost module only.
              ////      Other sources are given through output->mtransp.
              //// Here we remember yesterdays's monthly accumulated transpiration and evaporation,
              //// and subtract it from today's monthly sum.
              ////assert(lpj_to_fms_indices[cell] >= 0 && lpj_to_fms_indices[cell] < nlons*nlats);
              //tmp_q_ca_out[cell] = 
              //  grid[cell].output.mtransp - mtransp_yesterday[cell]
              //  + grid[cell].output.mevap - mevap_yesterday[cell]
              //  + grid[cell].output.mevap_lake - mevap_lake_yesterday[cell]
              //  + grid[cell].output.mevap_res - mevap_res_yesterday[cell];

              //mtransp_yesterday[cell] = grid[cell].output.mtransp;
              //mevap_yesterday[cell] = grid[cell].output.mevap;
              //mevap_lake_yesterday[cell] = grid[cell].output.mevap_lake;
              //mevap_res_yesterday[cell] = grid[cell].output.mevap_res;
              //tmp_q_ca_out[cell] = grid[cell].q_ca;

              //tmp_roughness_length_out[cell] = roughnesslength(grid[cell].standlist);

              // Soil Temperature of uppermost soil layer:
              //  temp[0] inside soil inside each stand of each cell.
              //for (i = 0; i < grid[cell].standlist->n; i++) {
              //  Real frac = ((struct stand *)(grid[cell].standlist->data)[i])->frac;
              //  if ((((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0])<-100){
              //    printf("ERR_CLIMBER: soiltemp too small %g", (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0]));
              //    (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0])=-100;
              //  }
              //  if ((((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0])>100){
              //     printf("ERR_CLIMBER: soiltemp too large %g", (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0]));
              //     (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0])=100;
              //  }
              //
              //  temp += (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0]) * frac;
              //  fracsum += frac;
              //}
              //if (grid[cell].laketemp<-50){
              //  printf("ERR_CLIMBER: laketemp too small %g", grid[cell].laketemp);
              //  grid[cell].laketemp=-50;
              //}
              //if (grid[cell].laketemp>100){
              //  printf("ERR_CLIMBER: laketemp too large %g", grid[cell].laketemp);
              //  grid[cell].laketemp=100;
              //}

              //for (i = 0; i < grid[cell].standlist->n; i++) {
              //  Real frac = ((struct stand *)(grid[cell].standlist->data)[i])->frac;
              //  if ((((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0])<-60){
              //    printf("ERR_CLIMBER: soiltemp too small %g", (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0]));
              //    (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0])=-60;
              //  }
              //  if ((((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0])>100){
              //    printf("ERR_CLIMBER: soiltemp too large %g", (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0]));
              //    (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0])=100;
              //  }
              //
              //  temp += (((struct stand *)(grid[cell].standlist->data)[i])->soil.temp[0]) * grid[cell].lakefrac;
              //  fracsum += grid[cell].lakefrac;
              //}

              //temp += grid[cell].laketemp * grid[cell].lakefrac;
              //fracsum += grid[cell].lakefrac;
              // // since we now also consider laketemp and lakefrac, fracsum should be just 1
              //tmp_surface_temp_out[cell] = temp / fracsum;
              //if (tmp_surface_temp_out[cell]<-100){
              //  printf("ERR_CLIMBER: final soiltemp too small %g", tmp_surface_temp_out[cell]);
              //  tmp_surface_temp_out[cell]=-100;
              //}
              // //tmp_albedo_out[cell] = albedo(&grid[cell]);
              // // TODO: use Erik's new albedo() parameterisation.
              // // It requires temperature and precipitation, which we also used for update_daily() above.
              //tmp_albedo_out[cell] = albedo(&grid[cell], tmp_temp_mean[cell], tmp_prec[cell]);
              //tmp_evap1_out[cell] = grid[cell].evap1_lastday;
              //tmp_dedt_out[cell] = grid[cell].dedt_lastday;
              //tmp_gc_out[cell] = grid[cell].gc;
              //tmp_exlpj_out[cell] = grid[cell].exlpj;

              //printf("EVAP1: %g \n", tmp_evap1_out[cell]);
            } /* if (!grid[cell].skip) */

            // collect runoff also from skipped cells!
            // Runoff: grid[cell].discharge.dfout contains the daily runoff [l/d]
            //   (grid[cell].daily.discharge is in [m3/s] = [t/s])
            //   It is updated in drain(), which is called above.
            tmp_runoff_out[cell] = grid[cell].discharge.dfout/86400.0; // [l/d] -> [l/s] = [kg/s]

          } /* loop over cells */

          /* now we can map the collected LPJ output to the FMS compute domain */
          //cpl_lpj_to_fms(tmp_carbon_flux_out, carbon_flux_out);
          //cpl_lpj_to_fms(tmp_q_ca_out, q_ca_out);
          //cpl_lpj_to_fms(tmp_roughness_length_out, roughness_length_out);
          //cpl_lpj_to_fms(tmp_surface_temp_out, surface_temp_out);
          //cpl_lpj_to_fms(tmp_albedo_out, albedo_out);
          //cpl_lpj_to_fms(tmp_evap1_out, evap1_out);
          //cpl_lpj_to_fms(tmp_dedt_out, dedt_out);
          //cpl_lpj_to_fms(tmp_gc_out, gc_out);
          //cpl_lpj_to_fms(tmp_exlpj_out, exlpj_out);
          cpl_lpj_to_fms(tmp_runoff_out, runoff_out);
          dayofyear++;
          for(cell=0;cell<config.ngridcell;cell++)
          { 
            grid[cell].daily.tmax=-1e38;
            grid[cell].daily.tmin=1e38;
            grid[cell].output.daily.rh=grid[cell].daily.prec=grid[cell].daily.swdown=grid[cell].daily.lwnet=grid[cell].daily.windspeed=grid[cell].daily.temp=grid[cell].VPD=0;
            //foreachstand(stand,s,grid[cell].standlist)
            //  stand->dprec=0;
          }
        } /* if (goodnight) */ /* of 'foreachdayofmonth */

        if (monthend) {
          for(cell=0;cell<config.ngridcell;cell++)
          {
            if(config.river_routing)
            {
              if(grid[cell].output.mdischarge<0)
                grid[cell].output.mdischarge=0;
              if(grid[cell].discharge.next<0)
                grid[cell].output.adischarge+=grid[cell].output.mdischarge;
              grid[cell].output.mdischarge*=1e-9/ndaymonth[month]; /* daily mean discharge per month in 1.000.000 m3 per cell */
              grid[cell].output.mres_storage*=1e-9/ndaymonth[month]; /* hb 5-11-09 mean monthly reservoir storage in 1.000.000 m3 per cell */
            }
            if(!grid[cell].skip)
              update_monthly(grid+cell,getmtemp(input.climate,&grid[cell].climbuf,
                                                cell,month),getmprec(input.climate,&grid[cell].climbuf,
                                                                     cell,month),month);
#ifdef DEBUG
            printcell(grid+cell,1,ncft,input.landuse!=NULL,TRUE);
#endif
          } /* of 'for(cell=0;...)' */

#ifdef IMAGE
          if(year>=config.firstyear-istimber*10)
#else
            if(year>=config.firstyear)
#endif
              /* write out monthly output */
              fwriteoutput_monthly(output,grid,month,year,&config);

        } /* if (monthend) */ /* of 'foreachmonth */

        if (silvester) { /* from iterateyear_river() */
          for(cell=0;cell<config.ngridcell;cell++)
          {
            if(!grid[cell].skip)
            {
              update_annual(grid+cell,npft,ncft,popdens,year,(config.prescribe_landcover!=NO_LANDCOVER) ? getlandcover(input.landcover,cell) : NULL,TRUE,intercrop,&config);
#ifdef SAFE
              check_fluxes(grid+cell,year,cell,&config);
#endif

#ifdef DEBUG
              if(year>config.firstyear)
              {
                printf("year=%d\n",year);
                printf("cell=%d\n",cell);
                printcell(grid+cell,1,ncft,input.landuse!=NULL);
              }
#endif
              if(config.equilsoil)
              {

                if(config.nspinup>veg_equil_year &&
                   year==config.firstyear-config.nspinup+veg_equil_year && !config.from_restart)
                  equilveg(grid+cell);

                if(config.nspinup>soil_equil_year &&
                   year==config.firstyear-config.nspinup+soil_equil_year && !config.from_restart)
                  equilsom(grid+cell,npft+ncft,config.pftpar,TRUE);

                if(config.nspinup>soil_equil_year &&
                   (year==config.firstyear-config.nspinup+soil_equil_year) && !config.from_restart)
                  equilsom(grid+cell,npft+ncft,config.pftpar,FALSE);

              }

            }

            grid[cell].output.surface_storage=grid[cell].discharge.dmass_lake+grid[cell].discharge.dmass_river;
            grid[cell].aprec_poem_last=grid[cell].aprec_poem;
            if(grid[cell].ml.dam)
            {
              grid[cell].output.surface_storage+=grid[cell].ml.resdata->dmass;
              for(i=0;i<NIRRIGDAYS;i++)
                grid[cell].output.surface_storage+=grid[cell].ml.resdata->dfout_irrigation_daily[i];
            }
          } /* of for(cell=0,...) */
#ifdef IMAGE
          if(year>=config.firstyear-istimber*10)
#else
            if(year>=config.firstyear)
#endif
            {
              /* write out annual output */
              fwriteoutput_annual(output,grid,year,&config);
              fwriteoutput_pft(output,grid,npft,ncft,year,&config);
            }

        } /* if (silvester) */ /* end iterateyear_river() */
      } /* end iterateyear_river */

    if (silvester) { /* from iterate() */
      if(year>=config.firstyear)
      closeoutput_yearly(output,&config);

      /* calculating total carbon and water fluxes collected from all tasks */
      cflux_total=flux_sum(&flux,grid,&config);
      if(config.rank==0)
      {
        /* output of total carbon flux and water on stdout on root task */
        printflux(flux,cflux_total,year,&config);
        if(output->method==LPJ_SOCKET && output->socket!=NULL &&
           year>=config.firstyear)
          output_flux(output,flux);
        fflush(stdout); /* force output to console */
#ifdef SAFE
        check_balance(flux,year,&config);
#endif
      }
#ifdef IMAGE
      if(year>=config.start_imagecoupling)
      {
        /* send data to IMAGE */
#ifdef DEBUG_IMAGE
        if(config.rank==0)
        {
          printf("sending data to image? year %d startimagecoupling %d\n",
                 year,config.start_imagecoupling);
          fflush(stdout);
        }
#endif
        if(send_image_data(&config,grid,input.climate,npft))
          fail(SEND_IMAGE_ERR,FALSE,
               "Problem with writing maps for transfer to IMAGE");
      }
#endif
      if(iswriterestart(&config) && year==config.restartyear)
        fwriterestart(grid,npft,ncft,year,config.write_restart_filename,&config); /* write restart file */
    } /* if (silvester) */

    yesterday = dayofmonth;

  } /* of 'for(year=...)' */

#ifdef STORECLIMATE
  if(config.nspinup && (config.lastyear<input.climate->firstyear || year<input.climate->firstyear))
  {
    /* restore climate data pointers to initial data */
    input.climate->data=data_save;
    freeclimatedata(&store); /* free data not used anymore */
  }
#endif
  return /*year*/;
}


#ifdef __cplusplus
extern "C"
#endif
void lpj_end_() {

  /* Simulation has finished */
  fcloseoutput(output,&config);
  if(isroot(config))
    puts((year>config.lastyear) ? "Simulation ended." : "Simulation stopped.");
  /* free memory */
  freeinput(input,isroot(config));
  freegrid(grid,config.npft[GRASS]+config.npft[TREE],&config);
  if(isroot(config))
  {
    printf( (year>config.lastyear) ? "%s successfully" : "%s errorneously",progname);
    printf(" terminated, %d grid cells processed.\n",
           config.total
           );
  }
#ifdef IMAGE
  if(config.sim_id==LPJML_IMAGE)
    close_image(&config);
#endif
  freeconfig(&config);
  cpl_free(cpl);

  //free(mevap_yesterday);
  //free(mevap_lake_yesterday);
  //free(minterc_yesterday);
  //free(mevap_res_yesterday);
  //free(mtransp_yesterday);
 printf("endclimber\n");

  return /*EXIT_SUCCESS*/ ;
}


// Calculate and return total amount of requested quantity per PE.
// Note: this function sums over the LPJ domain, and returns that local sum
// to the FMS. However, FMS only evaluates the global sum that it
// computes from the per-PE local sums. Thus we dont need to exchange the
// per-cell values via CPL grid shuffling.
#ifdef __cplusplus
extern "C"
#endif
void lpj_stock_pe_(const int *index, double *value) {
  int cell;
  *value = 0.0;
  switch(*index) {
  case 1: // water
    for(cell=0;cell<config.ngridcell;cell++) {
      if (!grid[cell].skip) *value += grid[cell].balance.totw;
    }
    break;
  case 2: // heat
    *value = 0.0; // for now
    break;
  case 3: // salt
    *value = 0.0;
    break;
  default: // unknown
    *value = 0.0;
    break;
  }
}
